<?php

/*
   This is the base class for all mod handling classes.
   It also provide logging services.

   Data:
      Common data and methods to assign values

   Methods:
      public function set_options( $options );
      private function create_logfile();
      public function write_eventlog();
      public function new_logevent( $string );
      public function add_logevent( $string );
*/

class modbase {

   const YES = "1";
   const NO = "0";

   const NOPATH = "-1";

   // values used for mod_status and logging
   const ERRORS = 'errors';
   const CANTDEL = 'cantdel';
   const FILEDEL = 'filedel';
   const CANTUPD = 'candupt';
   const INSTALLED = 'installed';
   const MODREM = 'modrem';
   const NOTINST = 'notinst';
   const OK2INSTALL = 'ok2install';
   const UPDATED = 'updated';

   protected $eventlog = '';

   // initializers
   protected $rootpath = '';
   protected $subroot ='';
   protected $modspath = '';
   protected $time_offset = 0;
   protected $sitever = 'standard';
   protected $currentuserdesc = '';
   protected $admtext = array();


   // options
   public $modlogfile = 'modmgrlog.txt';
   public $maxloglines = 2000;
   public $delete_partial = false;
   public $delete_installed = false;
   public $show_affected_files = true;
   public $show_AFnewfile = true;
   public $show_AFfilecopies = true;
   public $show_warnings = true;
   public $sortby = 0;
   public $fix_header = 1;
   public $use_striping = false;
   public $stripe_after = 3;
   protected $log_full_path = true;
   // basic mod info

   public $modname;
   public $version;
   public $description;
   public $wikipage = '';
   public $compress_names = false;
   protected $wikibase = "http://tng.lythgoes.net/wiki/index.php?title=";

   // reserve space for extended classes
   public $cfgpath = '';
   public $cfgfile = '';
   public $mod_status = '';

   // batch processing
   protected $batch_error = false;

	function __construct( $objinits ) {

      // copy init values to data area
      foreach( $objinits as $key => $value ) {
         if( $key == 'fpaths' ) {
            foreach( $value as $fpath => $fvalue ) {
               $this->$fpath = $fvalue;
            }
         }
         else {
            $this->$key = $value;
         }
      }

      // adjust some configuration values
      $this->modspath = rtrim( $this->modspath, "/" ) . "/";

      if( !empty( $this->admtext['error'] ) ) {
         $this->admtext['error'] = "<span class=\"msgerror\"><strong>{$this->admtext['error']}</strong></span>";
      }
      if( !empty( $this->admtext['okay']) ) {
         $this->admtext['okay'] = "<span class=\"msgapproved\"><strong>{$this->admtext['okay']}</strong></span>";
      }
	}

   public function set_options( $options ) {
      foreach( $options as $key => $value ) {
         if( isset( $this->$key ) ) {
            $this->$key = $value;
         }
      }
      // if visitor is mobile don't use fixed headers
      if( $this->sitever == 'mobile' ) $this->fix_header = 0;
   }

   private function create_logfile() {
      $rootpath = $this->rootpath;

      $content = "#### Mod Manager Log ".date ("D d M Y h:i:s A", time() + ( 3600 * $this->time_offset ) )." ####";
      return @file_put_contents( $this->modlogfile, $content );
   }

   protected function fix_eol( $instr )	{
      //$instr = preg_replace("#\r\r\n#","\r\n",$instr);         // Replace CRCRLF with CRLF
      //$instr = preg_replace("#([^\r])\n#","\\1\r\n",$instr);  // replace all standalone LFs with CRLF
      //$instr = preg_replace("#\r([^\n])#","\r\n\\1",$instr);  // replace all standalone CRs with CRLF
      //$instr = preg_replace("#([^\r])\n#","\\1\r\n",$instr);  // check again to catch double LFs
      //$instr = preg_replace("#\r([^\n])#","\r\n\\1",$instr);  // check again to catch double CRs
      //$instr = preg_replace("#[ \t]*\r\n#", "\r\n", $instr);  // remove trailing whitespace
      return preg_replace( "#(?:\r\n|\r\r\n|[\r\n])#", "\r\n", $instr );
      //return $instr;
   }

   public function write_eventlog() {
      //echo __FILE__,' ',__LINE__,' ',$this->modlogfile;exit;
      $admtext = $this->admtext;
      $mod_status = $this->mod_status;

      // if log file is missing start a new one
      if( !file_exists( $this->modlogfile ) ) {
         if( !$this->create_logfile() ) {
            $_SESSION['err_msg'] = "{$this->admtext['checkwrite']} {$this->admtext['cantwrite']} $this->modlogfile !";
            return false;
         }
      }
      //$_SESSION['err_msg'] = "  *Cannot create log file!";

      // import existing log lines
   	$lines = file( $this->modlogfile );

      // cut lines that exceed maxloglines user setting
   	while( $this->maxloglines && sizeof( $lines ) >= $this->maxloglines ) {
   		array_pop( $lines );
   	}

      // add dynamic results to event log
      $modstat1 = "<span class=\"hilight msgbold\">{$admtext[$mod_status]}</span>";
      $modstat2 = " <span class=\"hidden\">$mod_status</span>";

      $this->eventlog = str_replace( "zzzxxzzz", $modstat1, $this->eventlog );
      $this->eventlog = str_replace( "xxxzzxxx", $modstat2, $this->eventlog );

      // add new event log lines to $lines array
   	array_unshift( $lines, $this->eventlog."\n" );

   	$fp = @fopen( $this->modlogfile, "w" );
   	if( !$fp ) { 
		$_SESSION['err_msg'] = "{$this->admtext['checkwrite']} {$this->admtext['cannotopen']} $this->modlogfile ";
		return false;
	}

   	flock( $fp, LOCK_EX );
   	$linecount = 0;
   	foreach ( $lines as $line ) {
   		trim( $line );
   		if( $line )
   			fwrite( $fp, $line );
   		$linecount++;
   		if( $linecount == $this->maxloglines ) break;
   	}
   	flock( $fp, LOCK_UN );
   	fclose( $fp );
      $this->eventlog = '';
      return;
   }
   
   // begin logging for new event
   public function new_logevent( $string ) {
      //echo __FILE__,' ',__LINE__,' ',$string;exit;
      // get elements for enhanced header line
      $datetime = date( "D d M Y h:i:s A", time() + ( 3600 * $this->time_offset ) );
      if( $this->classID == 'installer' )
         $operation = $this->admtext['install'] = "<span class=\"msgbold\">{$this->admtext['install']}</span>";
      elseif( $this->classID == 'remover' )
         $operation = $this->admtext['uninstall'] = "<span class=\"msgbold\">{$this->admtext['uninstall']}</span>";
      elseif( $this->classID == 'cleaner')
         $operation = $this->admtext['cleanup'] = "<span class=\"msgbold\">{$this->admtext['cleanup']}</span>";
      elseif( $this->classID == 'editor' )
         $operation = $this->admtext['edparams'] = "<span class=\"msgbold\">{$this->admtext['edparams']}</span>";
      elseif( $this->classID == 'deleter' )
         $operation = $this->admtext['delete'] = "<span class=\"msgbold\">{$this->admtext['delete']}</span>";
      else
         $operation = '';

      $modname = $this->modname;
      $modversion = $this->version;

      $this->eventlog = $datetime.' | '.$operation.' | '.$modname.' '.$modversion.' | zzzxxzzz | '.$this->currentuserdesc.' xxxzzxxx';
      $this->add_logevent( $string);
   }

   public function add_logevent( $string ) {
      $this->eventlog .= "<br />&nbsp;&nbsp;&nbsp;&nbsp;*&nbsp;".$string;
   }
}
?>