<?php
/* Gen III Editor for mod parameters

   Methods:
      public function show_editor( $cfgpath );
      // note: $param is an array
      public function update_parameter( $param );
      public function restore_parameter( $param );
*/

include_once 'modparser.class.php';

class modeditor extends modparser {

   const UPDATEP = 6;
   const RESTOREP = 7;

   protected $classID = 'editor';

   function __construct( $objinits ) {
      // pass argument thru to to base class
      parent::__construct( $objinits );
   }

// need to show filename where parameters are being edited

   public function show_editor( $cfgpath ) {
      $this->cfgpath = $cfgpath;
      $this->cfgfile = $cfgfile = pathinfo( $cfgpath, PATHINFO_BASENAME );
      $this->mod_status = '';
      $admtext = $this->admtext;

      $this->parse( $this->cfgpath );

      $tags = $this->tags;

      // start logging this event
      $this->new_logevent( "{$admtext['editing']} <strong>$cfgfile</strong>" );
      if( count( $this->parse_errors ) ) {
         foreach( $this->parse_errors as $parse_err ) {
            if( $parse_err['text'] == 'nocfgfile' ) {
               $this->mod_status = self::ERRORS;
               $this->show_log_errors = true;
               $this->add_logevent("<span class=\"msgerror\">$cfgfile</span> <span class=\"hilighterr msgbold\">{$admtext['nocfgfile']}</span>");
               $this->write_eventlog();
               return false;
            }
         }
      }
      // build params array
      $i = 0; // params array index
      $params = array();
      for( $j=0; isset( $tags[$j] ); $j++ ) {

/*************************************************************************
         TARGETS
*************************************************************************/
         if( $tags[$j]['name'] == 'target' ) {
            if( $tags[$j]['arg1'] == 'files') continue;
            $active_target_file = '';
            // get target file path info
            $path = in_array( $tags[$j]['arg1'], $this->configs ) ? $this->subroot : $this->rootpath;
            $active_target_file = $path.$tags[$j]['arg1'];
            if( !file_exists( $active_target_file ) ) continue;
            $target_file_contents = file_get_contents( $active_target_file );
            continue;
         } // target
/*************************************************************************
         PARAMETERS
*************************************************************************/
         elseif( $tags[$j]['name'] == 'parameter' ) {
            //print_r( $tags[$j] );exit;
            if( !empty( $active_target_file) ) {

               // get current variable value from target file
               $varname = str_replace( "$", "\\$", $tags[$j]['arg1'] );
               $varname = str_replace( "[", "\\[", $varname );
               $varname = str_replace( "]", "\\]", $varname );

               // get current value from target file
               $reg_exp = "#(".$varname."\\s*=\\s*)(['\"]?)([^;]*);#";
               if( !preg_match( $reg_exp, $target_file_contents, $matches ) ) {
                  $this->mod_status = self::ERRORS;
                  $this->show_log_errors = true;
                  $this->add_logevent("<span class=\"msgerror\">{$admtext['tgtfile']} $active_target_file</span> <span class=\"hilighterr msgbold\">{$admtext['noparam']}: {$tags[$j]['arg1']}</span>");
                  $this->write_eventlog();
                  return false;
               }

               // keep track of surrounding quotes
               $quotes = 0;
               if( !empty( $matches[2]) ) {
                  if( $matches[2] == "'" ) $quotes = 1;
                  elseif( $matches[2] == '"' ) $quotes = 2;
               }

               // get rid of surrounding quotes for processing;
               $matches[3] = trim( $matches[3], "'\"" );

               // escape internal single quotes
               $matches[3] = str_replace( "'", "\\'", $matches[3] );

               $params[$i]['val'] = $matches[3];
               $params[$i]['def'] = $tags[$j+1]['arg2'];
               $params[$i]['tgt'] = $active_target_file;
               $params[$i]['cfg'] = $cfgpath;
               $params[$i]['param'] = $tags[$j]['arg1'];
               $params[$i]['quot'] = $quotes;
               $params[$i++]['label'] = $tags[$j+1]['arg1'];
                //print_r( $params );exit;
            }
         } // tag == parameter
      } // build params array
      unset( $target_file_contents );

      if( !count( $params ) ) {
         $this->mod_status = self::ERRORS;
         $this->show_log_errors = true;
         $this->add_logevent("<span class=\"msgerror\">$cfgfile</span> <span class=\"hilighterr msgbold\">{$admtext['cantupd']}: $varname</span>");
         $this->write_eventlog();
         return false;
      }
/*************************************************************************
SHOW TITLE BAR
*************************************************************************/
      // fixed or scrolling header/filter bar ?
      $tbclass = $this->fix_header == self::YES ? "tbar-fixed" : "tbar-scroll";
      $tableclass = $this->fix_header == self::YES ? "m3table-fixed" : "m3table-scroll";

      // fix for template 7
 /*     if( $this->templatenum == 7 ) {
         $tbclass = $tbclass.$this->templatenum;
         $tableclass = $tableclass.$this->templatenum;
      } */
      echo "
<table id=\"tbar\" class=\"normal lightback $tbclass\">
   <tr>
      <td style=\"padding-top:5;\">
         <div class=\"mmsubhead\">{$admtext['edopts']}: $cfgfile</div>
	 </td>
   </tr>
</table>

<table id=\"m3table\" class=\"normal lightback $tableclass\">
<tr>
<td>";
      // show the parameters editing form
      for( $i=0; isset( $params[$i] ); $i++ ) {
         $relpath = str_replace( $this->rootpath, '', $params[$i]['tgt'] );
         echo "
<form method=\"post\" action=\"\">
<table class=\"normal tfixed\">
<tr>
   <td class=\"databack edpanel mmleftcol\">
      {$params[$i]['label']}
   </td>
   <td class=\"databack edpanel mmrightcol\">
      <div>$relpath: {$params[$i]['param']}</div>
      <textarea class=\"w100\" name=\"param[val]\">{$params[$i]['val']}</textarea>
      <input type=\"hidden\" name=\"param[mod]\" value=\"$this->modname\" />
      <input type=\"hidden\" name=\"param[version]\" value=\"$this->version\" />
      <input type=\"hidden\" name=\"param[def]\" value=\"{$params[$i]['def']}\" />
      <input type=\"hidden\" name=\"param[tgt]\" value=\"{$params[$i]['tgt']}\" />
      <input type=\"hidden\" name=\"param[cfg]\" value=\"{$params[$i]['cfg']}\" />
      <input type=\"hidden\" name=\"param[param]\" value=\"{$params[$i]['param']}\" />
      <input type=\"hidden\" name=\"param[quot]\" value=\"{$params[$i]['quot']}\" />
      <div class=\"edbuttonbar\">
         <button type=\"submit\" name=\"submit\" value=\"pUpdate\">{$admtext['update']}</button>&nbsp;&nbsp;
         <button type=\"submit\" name=\"submit\" value=\"pRestore\">{$admtext['restore']}</button>&nbsp;&nbsp;
      </div>
   </td>
</tr>
</table>
</form>";
      } // each parameter

      echo "
<tr>
<td>
<div class=\"lightback edreturn\">
   <form method=\"post\" action=\"\">
            <button type=\"submit\" name=\"submit\" value=\"pCancel\">{$admtext['return']}</button>
   </form>
</div>
</td>
</tr>
</table>";
      return true;
   } // show_editor

   public function update_parameter( $param ) {
      //echo __FILE__,' ',__LINE__;print_r($param);exit;
      $admtext = $this->admtext;
      $this->modname = $param['mod'];
      $this->version = $param['version'];
      $this->new_logevent( "{$this->admtext['updparam']} {$param['param']}: <span class=\"msgbold\">{$param['tgt']}</span> {$this->admtext['formodname']} <span class=\"msgbold\">{$param['mod']}</span>"  );
      $this->cfgpath = $param['cfg'];
      $this->cfgfile = pathinfo( $param['cfg'], PATHINFO_BASENAME );
      if( !is_writable( $param['cfg'] ) ) {
         $this->mod_status = self::ERRORS;
         $this->add_logevent( "<span class=\"msgerror\">{$admtext['cantupd']}</span> <span class=\"hilighterr msgbold\">{$param['cfg']} %parameter:% {$param['param']}</span>" );
         $this->add_logevent( "{$admtext['fileperms']} ({$param['cfg']})" );
         $this->write_eventlog();
         header( "Location: admin_showmodslog.php" );
         exit;
      }
      if( !is_writable( $param['tgt'] ) ) {
         $this->mod_status = self::ERRORS;
         $this->add_logevent( "<span class=\"msgerror\">{$admtext['cantupd']}</span> <span class=\"hilighterr msgbold\">{$param['tgt']} %parameter:% {$param['param']}</span>" );
         $this->add_logevent( "{$admtext['fileperms']} ({$param['tgt']})" );
         $this->write_eventlog();
         header( "Location: admin_showmodslog.php" );
         exit;
      }
      $success = 0;
      $target_file_contents = file_get_contents( $param['tgt'] );

      // prepare original quoting around variable value
      $quotes = '';
      if( $param['quot'] == 1 ) $quotes = "'";
      elseif( $param['quot'] == 2 ) $quotes = '"';

      // remove any surrounding single or double quotes from new parameter value
      $varval = trim( $param['val'], "\"'" );
      //echo __LINE__,' varval = ', $varval,"<br />";
      //escape internal single quotes
      $varval = str_replace( "'", "\\'", $varval );
      //echo __LINE__,' varval = ', $varval,"<br />";
      // add original quoting
      $varval = $quotes.$varval.$quotes;
      //echo __LINE__,' varval = ', $varval,"<br />";exit;

      // escape $ sign and brackets in variable name
      $varname = str_replace( "$", "\\$", $param['param'] );
      $varname = str_replace( "[", "\\[", $varname );
      $varname = str_replace( "]", "\\]", $varname );
      //echo __FILE__,' ',__LINE__,' ',$param['param'];exit;
      $reg_exp = "#(".$varname."\\s*=\\s*)([^;]*);#";
      if( !preg_match( $reg_exp, $target_file_contents, $matches ) ) {
         $this->add_logevent( "<span class=\"msgerror\">{$admtext['cantupd']}</span> <span class=\"hilighterr msgbold\">{$param['tgt']} %parameter:% {$param['param']}</span>" );
         $this->mod_status = self::CANTUPD;
      }
      else {
         $success++;
         $this->add_logevent( "<span class=\"msgapproved\">{$param['param']} {$this->admtext['updated']}</span> ($varval)" );
         $this->mod_status = self::UPDATED;
      }
      $target_file_contents = str_replace( $matches[0], $matches[1]."$varval;", $target_file_contents );
      @file_put_contents( $param['tgt'], $target_file_contents );
      unset( $target_file_contents );

      $config_file_contents = file_get_contents( $param['cfg'] );

      // update inserted value to avoid bad target errors
      $config_file_contents = str_replace( $matches[0], $matches[1]."$varval;", $config_file_contents );

      // update the %parameter line val for backward compatibility
      $regex = '#%parameter:'.$varname.':'.'[^%]*%#';
      $parval = trim( $param['val'], "\"'" );
      $replacement = "%parameter:".$param['param'].":".$parval."%";
      $config_file_contents = preg_replace( $regex, $replacement, $config_file_contents );

      @file_put_contents( $param['cfg'], $config_file_contents );
      $this->write_eventlog();
      return $success;
   }

   public function restore_parameter( $param ) {
      //print_r( $param ); exit;
      $admtext = $this->admtext;
      $this->mod_status = self::ERRORS;
      $this->modname = $param['mod'];
      $this->version = $param['version'];
      $this->new_logevent( "{$this->admtext['restparam']} {$param['param']}: <span class=\"msgbold\">{$param['tgt']}</span> {$this->admtext['formodname']} <span class=\"msgbold\">{$param['mod']}</span>"  );
      $this->cfgpath = $param['cfg'];
      $this->cfgfile = pathinfo( $param['cfg'], PATHINFO_BASENAME );
      if( !is_writable( $param['cfg'] ) ) {
         $this->add_logevent( "<span class=\"msgerror\">{$admtext['cantupd']}</span> <span class=\"hilighterr msgbold\">{$param['cfg']} %parameter:% {$param['param']}</span>" );
         $this->add_logevent( "{$admtext['fileperms']} ({$param['cfg']})" );
         $this->write_eventlog();
         header( "Location: admin_showmodslog.php" );
         exit;
      }
      if( !is_writable( $param['tgt'] ) ) {
         $this->add_logevent( "<span class=\"msgerror\">{$admtext['cantupd']}</span> <span class=\"hilighterr msgbold\">{$param['tgt']} %parameter:% {$param['param']}</span>" );
         $this->add_logevent( "{$admtext['fileperms']} ({$param['tgt']})" );
         $this->write_eventlog();
         header( "Location: admin_showmodslog.php" );
         exit;
      }
      $success = 0;

      $target_file_contents = file_get_contents( $param['tgt'] );

      // remove surrounding single or double quotes from default parameter value
      $varval = trim( $param['def'], "\"'" );

      // escape internal single quotes
      $varval = str_replace( "'", "\\'", $varval );

      // add original surrounding quotes if any
      $quotes = '';
      if( $param['quot'] == 1 ) $quotes = "'";
      elseif( $param['quot'] == 2 ) $quotes = '"';
      $varval = $quotes.$varval.$quotes;

      // escape $ sign and brackets in variabel name
      // should already have been trimmed and escaped!!
      $varname = str_replace( "$", "\\$", $param['param'] );
      $varname = str_replace( "[", "\\[", $varname );
      $varname = str_replace( "]", "\\]", $varname );

      $reg_exp = "#([^\w]".$varname."\\s*=\\s*)([^;].*);#";
      if( !preg_match( $reg_exp, $target_file_contents, $matches ) ) {
         $this->add_logevent( "<span class=\"msgerror\">{$admtext['cantupd']}</span> <span class=\"hilighterr msgbold\">{$param['param']}</span>" );
         $this->mod_status = self::CANTUPD;
      }
      else {
         $success++;
         $this->add_logevent( "<span class=\"msgapproved\">{$param['param']} {$this->admtext['updated']}</span> ($varval)" );
         $this->mod_status = self::UPDATED;
      }

      $target_file_contents = str_replace( $matches[0], $matches[1]."$varval;", $target_file_contents );
      @file_put_contents( $param['tgt'], $target_file_contents );
      unset( $target_file_contents );

      $config_file_contents = file_get_contents( $param['cfg'] );

      // update inserted value to avoid bad target errors
      $config_file_contents = str_replace( $matches[0], $matches[1]."$varval;", $config_file_contents );

      // update the %parameter line val for backward compatibility
      $regex = '#%parameter:'.$varname.':'.'[^%]*%#';
      $varval = trim( $varval, "\"'" );
      $replacement = "%parameter:".$param['param'].":".$varval."%";
      $config_file_contents = preg_replace( $regex, $replacement, $config_file_contents );

      @file_put_contents( $param['cfg'], $config_file_contents );
      $this->write_eventlog();
      return $success;
   }
} // class modparser

?>
