<?php

/* Gen III Mod Installer Class

   Methods:
      public function install( $cfgpath );
      public function batch_install( $cfgpathlist );

   Modified 5/5/2017 - cleaned up buffer pointer management - wfb
*/

include_once 'modparser.class.php';

class modinstaller extends modparser {

   protected $classID = "installer";

   public function __construct( $objinits ) {
      // pass arguments thru to base class
      parent::__construct( $objinits );
   }

   public function install( $cfgpath ) {

      $this->cfgpath = $cfgpath;
      $this->cfgfile = $cfgfile = pathinfo( $cfgpath, PATHINFO_BASENAME );
	  // Temporary removed the fix I made for when config file not found - Ken
	  //$this->modname = $cfgfile;	// prime modname to config file name in case file not found
      $this->mod_status = '';
      $admtext = $this->admtext;

      $this->parse( $cfgpath );

      $tags = $this->tags;
      $num_tags = count( $tags );

		$locations_required = 0;
		$locations_installed = 0;
      //$total_files = 0;

      $newfiles_required = 0;
      $newfiles_created = 0;
      $newfiles_excused = 0;

      $copyfiles_required = 0;
      $copyfiles_copied = 0;
      $copyfiles_excused = 0;

      //$copies_excused = 0;
      //$total_files_copied = 0;
      $num_errors = 0;

      $line = $this->admtext['line'];


      /* track
      required_locations vs installed_locations
      num_errors
      total_file vs total_files_created + files_excused
      total_copyfiles vs total_files_copied + files_excused
      */

      $this->new_logevent( "{$admtext['installing']} <strong>$cfgpath</strong>" );

      // if there were fatal parsing errors, log them and exit to the log
      if( count( $this->parse_errors ) ) {
         $this->modname = $cfgfile;
         $this->mod_status = self::ERRORS;
         foreach( $this->parse_errors as $parse_err ) {
            $idx = $parse_err['text'];
            $this->add_logevent("<span class=\"msgerror\">$cfgfile</span> <span class=\"hilighterr msgbold\">{$admtext[$idx]}</span>");
         }
         $this->write_eventlog();
         return false;
      }
      for( $j=0; $j<$num_tags; $j++ ) {
/*************************************************************************
         TARGETS
*************************************************************************/
         if( $tags[$j]['name'] == 'target' ) {
            // skip "files" section here -- copyfiles done below
            if( $tags[$j]['arg1'] == 'files') continue;
            // Flush current target file buffer;
            if( !empty( $active_target_file ) && !empty( $target_file_contents ) ) {
               file_put_contents( $active_target_file, $target_file_contents );
            }
            $fileoptional = false;
            $target_file_basename = '';
            $active_target_file = '';
            $target_file_contents = '';

            // get file path info
            $path = in_array( $tags[$j]['arg1'], $this->configs ) ? $this->subroot : $this->rootpath;
            $target_file_basename = $tags[$j]['arg1'];
            $active_target_file = $path.$tags[$j]['arg1'];

            $fileoptional = $tags[$j+1]['name'] == 'fileoptional' ? true : false;
            $logstring = "$line {$tags[$j]['line']}: <span class=\"tag\">%target:</span><span class=\"tgtfile\">$target_file_basename</span><span class=\"tag\">%</span>&nbsp;";

            if( !file_exists( $active_target_file ) ) {
               if( $fileoptional ) {
                  $logstring .= "{$admtext['optmissing']}&nbsp;<span class=\"msgapproved\">{$admtext['bypassed']}</span>";
                  $this->add_logevent( $logstring );
               }
               else {
                  $num_errors++;
                  $logstring .= "<span class=\"msgerror\">{$admtext['tgtmissing']}</span>";
                  $this->add_logevent( $logstring );
               }
               $target_file_basename = '';
               $active_target_file = '';
               $target_file_contents = '';
               continue;
            }
            else {
               // try to open target file for processing
               $target_file_contents = $this->fix_eol( file_get_contents( $active_target_file ) );
               if( empty( $target_file_contents ) ) {
                  if( $fileoptional ) {
                     $logstring .= "{$admtext['optlocked']}&nbsp;<span class=\"msgapproved\">{$admtext['bypassed']}</span>";
                     $this->add_logevent( $logstring );
                  }
                  else {
                     $num_errors++;
                     $logstring .= "<span class=\"msgerror\">{$admtext['tgtlocked']}</span>";
                     $this->add_logevent( $logstring);
                     $target_file_basename = '';
                     $active_target_file = '';
                     $target_file_contents = '';
                  }
                  continue;
               }
               else {
                  $logstring .= "{$admtext['opened']}";
                  $this->add_logevent( $logstring );
                  continue;
               }
            }
         } // target
/*************************************************************************
         LOCATIONS
*************************************************************************/
         if( $tags[$j]['name'] == 'location' ) {
            $locations_required++;
            $logstring = "$line {$tags[$j]['line']}: <span class=\"tag\">%location:%</span>&nbsp;";

            // target file not opened for some reason
            if( empty( $active_target_file) ) {
               if( $fileoptional ) {
                  $locations_required--;
               }
               continue;
            }
            // target file is open and ready for modification
            else {
               $cfg_target_code = $tags[$j+1]['arg1'];
               $cfg_target_code_trimmed = trim( $cfg_target_code, " \t");
               $optag = $tags[$j+3]['raw']; // skip location %end:% tag
               $mod_code = $tags[$j+4]['arg1'];
               $logstring .= "($optag)&nbsp;";

               // create search strings to see if mod code is already installed
               // or if good target exists
               $block_op = false;
               switch( $optag ) {
                  case "%insert:before%":
                     $installed_string = $mod_code."\r\n";
                     $target_string = $cfg_target_code_trimmed;
                     $block_op = true;
                     break;
                  case "%insert:after%":
                     $installed_string = "\r\n".$mod_code;
                     $target_string = $cfg_target_code_trimmed;
                     $block_op = true;
                     break;
                  case "%replace:%":
                     $installed_string = $mod_code;
                     $target_string = $cfg_target_code_trimmed;
                     $block_op = true;
                     break;
                  case "%triminsert:before%":
                     $installed_string = $mod_code.$cfg_target_code;
                     $target_string = $cfg_target_code;
                     break;
                  case "%triminsert:after%":
                     $installed_string = $cfg_target_code.$mod_code;
                     $target_string = $cfg_target_code;
                     break;
                  case "%trimreplace:%":
                     $installed_string = $mod_code;
                     $target_string = $cfg_target_code;
                     break;
                  default:
                     break;
               }
               // if mod code already installed just accept it and move on
               // don't install this code again if another version of the mod already has
               if( false !== strpos( $target_file_contents, $installed_string ) ) {
                  $locations_installed++;
                  $logstring .= "<span class=\"msgapproved\">{$admtext['installed']}</span>";
                  $this->add_logevent( $logstring );
                  continue;
               }

               // if good target, set pointer to start of masssaged target code
               if( false === $i = strpos( $target_file_contents, $target_string ) ) {
                  $num_errors++;
                  $logstring .= "<span class=\"msgerror\">{$admtext['badtarget']}</span>";
                  $this->add_logevent( $logstring );
                  continue;
               }

               // set rest of target file pointers according to type of directive
               if( $block_op ) {
                  $fragment = false;
                  // set pointer to start of first line skipping any text or white space
                  // SELECT BLOCK OF TEXT IN TARGET FILE TO OPERATE ON
                  // set pointer ($p) to beginning of matched text from cfg and move it
                  // backwards back to end of CRLF or top of file, skipping white space
                  // but checking to see if we have a fragment
                  $p = $i;
                  while( $p > 0 ) {
                     $p--;
                     if( $target_file_contents[$p] == "\n" ) {
                        $p++;
                        break;
                     }
                     if( $target_file_contents[$p] != " " && $target_file_contents[$p] != "\t" ) {
                        $fragment = true;
                     }
                  }
                  $start = $p;

                  // set pointer to end of last line of target and move it forward to CRLF
                  // or end of file, skipping white space but checking for data between
                  // end of the target and the end of the block (CRLF) making it a fragment
                  $p = $i + strlen( $target_string ); // last position in target str
                  while( true ) {
                     if( !isset($target_file_contents[$p] ) ) { // EOF - no CRLF
                        break;
                     }
                     if( $target_file_contents[$p] == "\r" ) { // CRLF
                        break;
                     }
                     if( $target_file_contents[$p] != " " && $target_file_contents[$p] != "\t" ) { // not a whitespace char
                        $fragment = true;
                     }
                     $p++;
                  }
                  $end = $p;
                  $length = ( $end - $start ) + 1;

/*
                  // set pointer to end of last line skipping any text or white space
                  $k = ( $i + strlen( $target_string ) );
                  while( isset( $target_file_contents[$k] ) && ($target_file_contents[$k] != "\r" ) ) {
                     if( $target_file_contents[$k] != " " && $target_file_contents[$k] != "\t")
                        $fragment = true;
                  	$k++;
                  }
                  $end = $k;
                  $length = ( $end - $start ) + 1;
*/
                  // make sure %replace:% uses block
                  if( $fragment && $optag == "%replace:%" ) {
                     $num_errors++;
                     $logstring .= "<span class=\"msgerror\">{$admtext['no_frag']}</span>";
                     $this->add_logevent( $logstring );
                     continue;
                  }
               }
               else { // not changes to cfg code or target code - they must exactly match
                  $start = $i;
                  $length = strlen( $target_string );
                  $end = ( $start + $length );
               }
               // set values for replacement or insertion
               switch( $optag ) {
                  case "%insert:before%":
                     $code = $mod_code."\r\n";
                     $pos = $start;
                     $len = 0; // zero for insert at $pos; # for amount of tgt to replace
                     break;
                  case "%insert:after%":
                     $code = "\r\n".$mod_code;
                     $pos = $end;
                     $len = 0;
                     break;
                  case "%replace:%":
                     $code = $mod_code;
                     $pos = $start;
                     $len = $length;
                     break;
                  case "%triminsert:before%":
                     $code = $mod_code;
                     $pos = $start;
                     $len = 0;
                     break;
                  case "%triminsert:after%":
                     $code = $mod_code;
                     $pos = $end;  // use tgt measurement
                     $len = 0;
                     break;
                  case "%trimreplace:%":
                     $code = $mod_code;
                     $pos = $start;
                     $len = $length;
                     break;
                  default:
                     break;
               }

               // modify the target file contents
               $target_file_contents = substr_replace( $target_file_contents, $code, $pos, $len );

               // test for success
               if( false !== strpos( $target_file_contents, $code ) ) {
                  $locations_installed++;
                  $logstring .= "<span class=\"msgapproved\">{$admtext['installed']}</span>";
                  $this->add_logevent( $logstring );
                  continue;
               }
               else {
                  $num_errors++;
                  $logstring .= "<span class=\"msgerror\">{$admtext['badinstall']}</span>";
                  $this->add_logevent( $logstring );
                  continue;
               }
            } // open target file
         } // locations
/*************************************************************************
         FILE OPERATIONS
*************************************************************************/
         if( $tags[$j]['name'] == 'copyfile' || $tags[$j]['name'] == 'copyfile2' ) {
            $optional = '';
            if( false !== strpos($tags[$j]['arg1'], "@" ) ) {
               $optional = '@';
               $tags[$j]['arg1'] = str_replace( "@", "", $tags[$j]['arg1'] );
            }
            $copyfiles_required++;
            $source_file = $tags[$j]['arg1'];
            $source_path = $this->rootpath.$this->modspath.$source_file;

            //echo $source_path;exit;
            $filename = pathinfo( $source_path, PATHINFO_BASENAME );
            $destination_path = $tags[$j]['name'] == 'copyfile' ? $this->rootpath.$filename : $this->rootpath.$tags[$j]['arg2'];

            $logstring = "$line {$tags[$j]['line']}: %<span class=\"tag\">{$tags[$j]['name']}:</span>";
            if( $tags[$j]['name'] == 'copyfile') {
               $logstring .= "<span class=\"tgtfile\">$optional{$tags[$j]['arg1']}</span>%&nbsp;";
            }
            else {
               $logstring .= "<span class=\"tgtfile\">$optional{$tags[$j]['arg2']}</span>%&nbsp;";
            }

            if( !file_exists( $source_path ) ) {
               if( $optional ) {
                  $copyfiles_excused++;
                  $logstring .= "{$admtext['optsrcfilemissing']}&nbsp;<span class=\"msgapproved\">{$admtext['bypassed']}</span>";
                  $this->add_logevent( $logstring );
                  continue;
               }
               else {
                  $num_errors++;
                  $logstring .= "<span class=\"msgerror\">{$admtext['srcfilemissing']}</span>";
                  $this->add_logevent( $logstring );
                  continue;
               }
            }
         	if( @copy( $source_path, $destination_path ) === false ) {
               // failed copy excused?
               if( $optional ) {
                  $copyfiles_excused++;
                  $logstring .= "{$admtext['optnocopy']}&nbsp;<span class=\"msgapproved\">{$admtext['bypassed']}</span>";
                  $this->add_logevent( $logstring );
                  continue;
               }
               // failed copy
               else {
                  $num_errors++;
                  $logstring .= "<span class=\"msgerror\">{$admtext['notcopied']}</span>";
             		$this->add_logevent( $logstring );
                  continue;
               }
         	}
            else {
               // copy was successful
               $copyfiles_copied++;
               $logstring .= "<span class=\"msgapproved\">{$admtext['copied']}</span>";
               $this->add_logevent( $logstring);
               continue;
         	}
         }
         elseif( $tags[$j]['name'] == 'newfile' ) {
            $optional = '';
            if( false !== strpos($tags[$j]['arg1'], "@" ) ) {
               $optional = '@';
               $tags[$j]['arg1'] = str_replace( "@", "", $tags[$j]['arg1'] );
            }
            $newfiles_required++;
            $this->num_required++;
            $destination_file = $tags[$j]['arg1'];
            $logstring = "$line {$tags[$j]['line']}: <span class=\"tag\">%newfile:</span><span class=\"tgtfile\">$optional$destination_file</span><span class=\"tag\">%</span>&nbsp;";

            $newfile_code = '';
            for( $i=$j+1; $i<$num_tags; $i++ ) {
               if( $tags[$i]['name'] == 'code' ) {
                  $newfile_code = $tags[$i]['arg1'];
                  break;
               }
            }
            if( !empty( $newfile_code ) ) {
			  if( false !== @file_put_contents( $this->rootpath.$destination_file, $newfile_code ) ) {
            		$newfiles_created++;
                  $logstring .= "<span class=\"msgapproved\">{$admtext['created']}</span>";
            		$this->add_logevent( $logstring );
                  continue;
               }
               // optional file not created?
               elseif( $optional ) {
                  $newfiles_excused++;
                  $logstring .= "{$admtext['optnotcreated']}&nbsp;<span class=\"msgapproved\">{$admtext['bypassed']}</span>";
                  $this->add_logevent( $logstring );
                  continue;
               }
               else {
                  $num_errors++;
                  $logstring .= "<span class=\"msgerror\">{$admtext['notcreated']}</span>";
                  $this->add_logevent( $logstring );
                  continue;
               }
            }

         } // files
      } // tags processing loop

      // Everything processed, flush last buffer to file
      if( !empty( $active_target_file ) && !empty( $target_file_contents ) ) {
         file_put_contents( $active_target_file, $target_file_contents );
      }
      if( !$num_errors ) {

         if( $copyfiles_required == ( $copyfiles_copied + $copyfiles_excused ) && ( $newfiles_required == ( $newfiles_created + $newfiles_excused ) && ( $locations_required == $locations_installed ) ) ) {
            if( $copyfiles_copied == 0 && $copyfiles_excused == 0 && $newfiles_created == 0 && $newfiles_excused == 0 && $locations_installed == 0 ) {
               $status = self::OK2INSTALL;
               $class = "class=\"msgapproved\"";
            }
            else {
            $status = self::INSTALLED;
            $class = "class=\"msgapproved\"";
            }
         }
      }
      else {
         $status = self::ERRORS;
         $class = "class=\"msgerror\"";
         $this->batch_error = true;
      }
      $this->add_logevent("<span class=\"msgbold\">{$admtext['toterrors']}:</span> $num_errors");
      $this->add_logevent("$locations_required {$admtext['modsreq']}: $locations_installed {$admtext['modsinst']}" );
      $this->add_logevent("$copyfiles_required {$admtext['copiesreq']}: $copyfiles_copied {$admtext['filescopied']} and $copyfiles_excused {$admtext['excused']}" );
      $this->add_logevent("$newfiles_required {$admtext['newfilesreq']}: $newfiles_created {$admtext['nfcreated']} and $newfiles_excused {$admtext['excused']}");
		$this->add_logevent("{$admtext['status']}: <span $class>{$admtext[$status]}</span>");

      $this->mod_status = $status;
      $this->write_eventlog();
      return ( $status == self::OK2INSTALL || $status == self::INSTALLED );
   } // install

   public function batch_install( $cfgpathlist ) {
      //echo __FILE__,' ',__LINE__,'<br />';print_r( $cfgpathlist );exit;
      foreach( $cfgpathlist as $cfgpath ) {
         if( !$this->install( $cfgpath ) ) {
            $this->batch_error = true;
         };
      }
      return !$this->batch_error;
   }
} // class modparser

?>