<?php
/*
   Gen III mod manager class to display main mod listing
   Methods:
      public function list_mods();
      protected function format_status( $status_header, $status, $num_errors, $id );
      protected function get_affected_files( $id );
      private function format_affrows( $file_array, $label );
      protected function list_aff_files( $regex, $contents );

   Mofified 5/5/2017 - better buffer pointer management.
*/
include_once 'classes/modparser.class.php';

class modlister extends modparser {

   // filter
   const F_ALL = 0;
   const F_READY = 1;
   const F_INSTALLED = 2;
   const F_CLEAN = 3;
   const F_BADCFG = 4;

   // sorting
   const NAMECOL = "0";
   const FILECOL = "1";

   // processing
   const INSTALL = 1;
   const REMOVE = 2;
   const DELETE = 3;
   const CLEANUP = 4;
   const EDITP = 5;
   const UPDATEP = 6;
   const RESTOREP = 7;

   // status
   const OK2INST     = 'ok2inst';
   const PARTINST    = 'partinst';
   const INSTALLED   = 'installed';
   const NOTINST     = 'notinst';

   protected $classID = "lister";

   public $filter = self::F_ALL;
   public $afchange = array();
   public $afcreate = array();
   public $afcopy = array();
   public $afcopy2 = array();
   public $fbox_checked = false;

   public $parameters = 0;
   private $num_required = 0;
   private $mods_required = 0;
   private $copies_required = 0;
   private $newfiles_required = 0;
   private $num_installed = 0;
   private $mods_installed = 0;
   private $copies_installed = 0;
   private $newfiles_installed = 0;

   function __construct( $objinits ) {
      // pass arguments thru to base class
      parent::__construct( $objinits );
   }

   public function list_mods() {
      /*******************************************************************
      FILTERING
      *******************************************************************/
      $admtext = $this->admtext;
      // construct button line per filter setting
      $doselect = true;
      $lockit = "";
      $btnline = '';

      if( $this->filter != self::F_ALL ) {
         $cbchecked = $this->fbox_checked ? "checked" : "";
         $lockit .= "
            {$admtext['stayon']}&nbsp;&nbsp;<input type=\"checkbox\" id=\"stayon\" $cbchecked/>";
      }
      if( $this->filter == self::F_READY ) {
         $btnline .= "
         <button type=\"submit\" id=\"btnInstall\" class=\"msgapproved\" name=\"submit\" value=\"installall\">{$admtext['installall']}</button>
         &nbsp;&nbsp;
         <button type=\"submit\" id=\"btnDelete\" class=\"msgerror\" name=\"submit\" value=\"deleteall\" />{$admtext['deleteall']}</button>";
      }
      elseif( $this->filter == self::F_INSTALLED ) {
         $btnline .= "
         <button type=\"submit\" id=\"btnRemove\" class=\"msgapproved\" name=\"submit\" value=\"removeall\">{$admtext['removeall']}</button>";
      }
      elseif( $this->filter == self::F_CLEAN ) {
         $btnline .= "
         <button type=\"submit\" id=\"btnClean\" class=\"msgapproved\" name=\"submit\" value=\"cleanupall\" />{$admtext['cleanupall']}</button>";
         if( $this->delete_partial ) {
            $btnline .= "
            &nbsp;&nbsp;
            <button type=\"submit\" id=\"btnDelete\" class=\"msgerror\" name=\"submit\" value=\"deleteall\" />{$admtext['deleteall']}</button>";
         }
      }
      elseif( $this->filter == self::F_BADCFG ) {
         $btnline .= "
         <button type=\"submit\" id=\"btnDelete\" class=\"msgerror\" name=\"submit\" value=\"deleteall\" />{$admtext['deleteall']}</button>";
      }
      else {
         $btnline .= "
         &nbsp;&nbsp;{$admtext['choosefilter']}";
      }
      if( $this->filter != self::F_ALL ) {
         $selectboxes = "
         <button type=\"button\" id=\"selectAll\">
            {$admtext['selectall']}
         </button>
         &nbsp;&nbsp;
         <button type=\"button\" id=\"clearAll\">
            {$admtext['clearall']}
         </button>";
      }
      else {
         $selectboxes = '';
      }
      // list will be in file name order
      $modlisting = $this->get_modlist();
      if( $modlisting == self::NOPATH ) {
         $sysmsg = "{$admtext['cannotopendir']}: ".rtrim($this->modspath, "/")."!";
      }
      else {
         $this->admtext['nomods'] = str_replace( "xxx", "cfg", $this->admtext['nomods'] );
         $sysmsg = count( $modlisting ) ? '' : "{$this->admtext['nomods']}";
      }

      // test single mod
      //echo $this->rootpath;exit;
     //$modlisting = array( 'media_icons_mod_v10.1.0.1.cfg' => 'mod manager test');

      /*******************************************************************
      HANDLE SORTING
      *******************************************************************/
      $filesort = $namesort = '';
      if( $this->sortby == self::NAMECOL ) {
         // sort array by value (mod name) case-sensitive
         // use temporary array because we can't actually change file name
         $temp = array_map('strtolower', $modlisting);
         array_multisort( $temp, SORT_ASC, SORT_STRING, $modlisting );
         $filesort = "<a href=\"admin_modhandler.php?sort=".NAMECOL."\"><img src=\"img/tng_sort_asc.gif\"
            width=\"15\" height=\"8\" border=\"0\" alt=\"\" title=\"{$admtext['text_sort']}\" /></a>";
         $namesort = "<img src=\"img/tng_sort_desc.gif\"
            width=\"15\" height=\"8\" border=\"0\" alt=\"\" />";
      }
      else {
         // default sort is file name order
         //uksort( $mod_name_array, "strcasecmp" );
         $namesort = "<a href=\"admin_modhandler.php?sort=".FILECOL."\"><img src=\"img/tng_sort_asc.gif\"
            width=\"15\" eight=\"8\" border=\"0\" alt=\"\" title=\"{$admtext['text_sort']}\" /></a>";
         $filesort = "<img src=\"img/tng_sort_desc.gif\"
            width=\"15\" eight=\"8\" border=\"0\" alt=\"\" />";
      }
      $line = self::LINE;
      // Start table with headers <br>

      // set selected value in filter drop down list
      $s0 = $s1 = $s2 = $s3 = $s4 = '';
      ${'s'.$this->filter} = "selected";

      //adjust column widths for affected files
      //$status_width = $this->show_affected_files ? '40%' : '42%';
		$status_width = '42%';

      // fixed or scrolling header/filter bar ?
      $fbclass = $this->fix_header == self::YES ? "fbar-fixed" : "fbar-scroll";
      $mmlistclass = $this->fix_header == self::YES ? "mmlist-fixed" : "mmlist-scroll";

      // fix for template 7
      /*if( $this->templatenum == 7 ) {
         $fbclass = $fbclass.$this->templatenum;
         $mmlistclass = $mmlistclass.$this->templatenum;
      }*/

/*************************************************************************
SHOW FILTER BAR
*************************************************************************/
      echo "
<form action=\"admin_modhandler.php\" method=\"POST\">
<table id=\"fbar\" class=\"normal lightback $fbclass\">
   <tr>
      <td class=\"fieldnameback fieldname mmpadtop5\">
         <div class=\"float-left\">
         &nbsp;{$admtext['statusfilter']}:&nbsp;
         <select name=\"filter\" >
            <option $s0 value=".self::F_ALL.">{$admtext['all']}</option>
            <option $s1 value=".self::F_READY.">{$admtext['ready']}</option>
            <option $s2 value=".self::F_INSTALLED.">{$admtext['installed']}</option>
            <option $s3 value=".self::F_CLEAN.">{$admtext['partinst']}</option>
            <option $s4 value=".self::F_BADCFG.">{$admtext['cantinst']}</option>
         </select>
         <input type=\"submit\" name=\"newlist\" value=\"{$admtext['go']}\" />
         &nbsp;&nbsp;$lockit&nbsp;&nbsp;$selectboxes &nbsp;&nbsp; $btnline
         </div>
	 </td>
   </tr>
</table>";

/*************************************************************************
SHOW LISTING
*************************************************************************/
// uses bounding table to eliminate jumping during page load
      echo "
<table id=\"mmgrid\" class=\"lightback $mmlistclass\">
<tr class=\"databack\">
<td class=\"tngshadow\">

<table class=\"tfixed normal\">
   <tr>";
if ($this->filter != self::F_ALL) {
   echo "
		<th class=\"fieldnameback fieldname center colselct\">
         <div class=\"mminner\">&nbsp;{$admtext['choose']}</div>
      </th>";
}
   echo "
      <th class=\"fieldnameback fieldname center colmodnm\">{$admtext['modname']}&nbsp;&nbsp;$namesort</th>
      <th class=\"fieldnameback fieldname center colcfgnm\">{$admtext['cfgname']}&nbsp;&nbsp;$filesort</th>
      <th class=\"fieldnameback fieldname center colversn\">{$admtext['version']}</th>
      <th class=\"fieldnameback fieldname center colwiki\">
         <div class=\"mminner\">
            {$admtext['wiki']}
         </div>
      </th>
      <th class=\"fieldnameback fieldname center colstatus\">{$admtext['status']}</th>
      <th class=\"fieldnameback fieldname center colaflist\"><strong>{$admtext['aflist']}</strong></th>
   <tr>";

      $id = 0;
      $ix = 0;
      $dbx = 0;
      $dbclass = 'databackalt';
      $white = array( " ", "\t" );
      //echo __FILE__,' ',__LINE__;print_r( $modlisting );exit;
      if( !empty( $sysmsg ) ) {
         echo "
   <tr>
      <td colspan=\"6\" class=\"mmsysmsg\">
         $sysmsg
      </td>
   </tr>
   </table>";
   return;
      }
      foreach( $modlisting as $cfgfile => $modname ) {
         $this->cfgfile = $cfgfile;
         $cfgpath = $this->rootpath.$this->modspath.$cfgfile;
         $this->cfgpath = $cfgpath;
         $num_errors = 0;
         $this->init_status();
/* -------- block 1 removed ----------

         // save lists of files affected by this mod
            $this->afchange = array();
            $this->afcreate = array();
            $this->afcopy = array();
            $this->afcopy2 = array();
            if( false !== $config_contents = $this->fix_eol(file_get_contents( $cfgpath ) ) ) {
               $this->afchange = $this->list_aff_files( "#%target:#", $config_contents );
               $this->afcreate = $this->list_aff_files( "#%newfile:#", $config_contents);
               $this->afcopy = $this->list_aff_files( "#%copyfile:#", $config_contents );
               $this->afcopy2 = $this->list_aff_files( "#%copyfile2:[^:]*:#", $config_contents);
               unset( $config_contents );
               //print_r( $this->afchange );
               //print_r( $this->afcreate );
               //print_r( $this->afcopy );
               //print_r( $this->afcopy2 );
            }

----------block 1 */
         $this->parse( $cfgpath );
         $this->save_affected_files( $this->tags );
         if( !count( $this->parse_errors ) ) {
            /*************************************************************************
            SET STATUS ELEMENTS FOR THIS MOD
            *************************************************************************/
            $this->opstate = array();
            $this->num_required = 0;
            $this->mods_required = 0;
            $this->copies_required = 0;
            $this->newfiles_required = 0;
            $this->num_installed = 0;
            $this->mods_installed = 0;
            $this->copies_installed = 0;
            $this->newfiles_installed = 0;
            $this->parameters = 0;
            $sidx = 0;  // State array index
            $tags = $this->tags;

            // process parse table
            for( $j=0; isset( $tags[$j] ); $j++ ) {
               /*************************************************************************
               TARGETS
               *************************************************************************/
               if( $tags[$j]['name'] == 'target' ) {
                  $fileoptional = false;
                  $active_target_file = '';

                  if( $tags[$j]['arg1'] != 'files' ) {
                     $fileoptional = $tags[$j+1]['name'] == 'fileoptional' ? true : false;

                     // get file path info
                     $path = in_array( $tags[$j]['arg1'], $this->configs ) ? $this->subroot : $this->rootpath;
                     $target_file_basename = $tags[$j]['arg1'];
                     $active_target_file = $path.$tags[$j]['arg1'];
                     $statstring = "$line {$tags[$j]['line']}: %<span class=\"tag\">target:</span><span class=\"tgtfile\">$target_file_basename</span>%&nbsp;";

                     // target file does not exist
                     if( !file_exists( $active_target_file ) ) {
                        $active_target_file = '';
                        if( $fileoptional ) {
                           $statstring .= "{$admtext['optmissing']}&nbsp;<span class=\"msgapproved\">{$admtext['willbypass']}</span>";
                           $this->set_status( $statstring );
                           continue;
                        }
                        else {
                           $num_errors++;
                           $statstring .= "<span class=\"hilighterr msgbold\">{$admtext['tgtmissing']}</span>";
                           $this->set_status( $statstring );
                           continue;
                        }
                     }

                     // target file exist - check if we can write to it
                     if( !is_writable( $active_target_file ) ) {
                        $active_target_file = '';
                           $num_errors++;
                           $statstring .= "<span class=\"hilighterr msgbold\">{$admtext['notwrite']}</span>";
                           $this->set_status( $statstring );
                           continue;
                     }
					 
                     // Get target file contents
                     elseif( !empty( $active_target_file ) ) {
                        $target_file_contents = $this->fix_eol( file_get_contents( $active_target_file ) );

                        // File exists - access error
                        if( empty( $target_file_contents ) ) {
                           $num_errors++;
                           $active_target_file = '';
                           $statstring .= "<span class=\"msgerror\">{$admtext['tgtlocked']}</span>";
                           $this->set_status( $statstring );
                           continue;
                        }
                        // Target File OK 
                        else {
                           $this->set_status( $statstring );
                           continue;
                        }
                     }
                  }
               } // targets

               /*************************************************************************
               PARAMETERS
               *************************************************************************/
               elseif( $tags[$j]['name'] == 'parameter' ) {
                  if( empty( $active_target_file ) ) continue;
                  $this->parameters++;
                  $statstring = "$line {$tags[$j]['line']}: %<span class=\"tag\">parameter:{$tags[$j]['arg1']}:{$tags[$j]['arg2']}</span>%&nbsp;";
                  $this->set_status( $statstring );
               }
               /*************************************************************************
               LOCATIONS
               *************************************************************************/
               // simplify finding target code -- do not attempt to deal with
               // leading or trailing whitespace

               elseif( $tags[$j]['name'] == 'location' ) {
                  if( empty( $active_target_file) ) continue;
                  $this->num_required++;
                  $this->mods_required++;
                  $optag = $tags[$j+3]['raw']; // skip %end:% tag
                  $target_code = $tags[$j+1]['arg1'];
                  $target_code_trimmed = trim( $target_code, " \t" );
                  $new_code = $tags[$j+4]['arg1'];
                  $statstring = "$line {$tags[$j]['line']}: <span class=\"tag\">%location:%</span>&nbsp;";

                  if( $optag == "%insert:before%" ) {
                     $installed_code = $new_code."\r\n";
                     $good_target = $target_code_trimmed;
                  }
                  elseif( $optag == "%insert:after%" ) {
                     $installed_code = "\r\n".$new_code;
                     $good_target = $target_code_trimmed;
                  }
                  elseif( $optag == "%replace:%" ) {
                     $installed_code = $new_code;
                     $good_target = $target_code_trimmed;
                  }
                  elseif( $optag == "%triminsert:before%" ) {
                     $installed_code = $new_code.$target_code;
                     $good_target = $target_code;
                  }
                  elseif( $optag == "%triminsert:after%" ) {
                     $installed_code = $target_code.$new_code;
                     $good_target = $target_code;
                  }
                  elseif( $optag == "%trimreplace:%" ) {
                     $installed_code = $new_code;
                     $good_target = $target_code;
                  }
                  // already installed?
                  $already_installed = substr_count( $target_file_contents, $installed_code );
                  if( $already_installed ) {
                     $this->num_installed++;
                     $this->mods_installed++;
                     $statstring .= $admtext['installed'];
                     $this->set_status( $statstring );
                     continue;
                  }

                  // not installed -- good target?
                  $num_targets = substr_count( $target_file_contents, $good_target );

                  // inserting new code will result in 'not unique' condition
                  if( $num_targets == 1 ) {
                     if( $optag != '%replace:%' && $optag != '%trimreplace:%' ) {
                        if( false !== strpos( $new_code, $target_code ) ) {
                           $num_errors++;
                           $statstring .= "<span class=\"hilighterr msgbold\">{$admtext['notunique']}</span>";
                           $this->set_status( $statstring );
                           continue;
                        }
                     }
                  }
                  // more than one target found (not unique)
                  if( $num_targets > 1 ) {
                     $num_errors++;
                     $statstring .= "<span class=\"hilighterr msgbold\">{$admtext['notunique']}</span>";
                     $this->set_status( $statstring );
                     continue;
                  }
                  // no target code found (bad target)
                  if( $num_targets == 0 ) {
                     $num_errors++;
                     $statstring .= "<span class=\"hilighterr msgbold\">{$admtext['badtarget']}</span>";
                     $this->set_status( $statstring );
                     continue;
                  }

                  // check for %replace:% fragment error
                  if( $optag == "%replace:%" ) {
                     $is_fragment = false;
                     if( false !== $i = strpos( $target_file_contents, $good_target ) ) {
                        // check for non-whitespace before targeted code on first line
                        $p = $i;
                        while( $p > 0 ) {
                           $p--;
                           if( $target_file_contents[$p] == "\n" ) {
                              break;
                           }
                           if( $target_file_contents[$p] != " " && $target_file_contents[$p] != "\t" ) {
                              $fragment = true;
                              break;
                           }
                        }
                        if( !$is_fragment ) {
                           // check for non-white space after target code on last line
                           $p = $i + strlen( $good_target ); // last position in target str
                           while ( true ) {
                              if( !isset($target_file_contents[$p] ) ) { // EOF - no CRLF
                                 break;
                              }
                              if( $target_file_contents[$p] == "\r" ) { // CRLF
                                 break;
                              }
                              if( $target_file_contents[$p] != " " && $target_file_contents[$p] != "\t" ) { // not a whitespace char
                                 echo $target_file_contents[$p];
                                 $is_fragment = true;
                                 break;
                              }
                              $p++;
                           }
                        }
                     }
                     if( $is_fragment ) {
                     //echo __LINE__ .": ".$cfgfile." ";exit;
                        $num_errors++;
                        $statstring .= "<span class=\"msgerror\">{$admtext['no_frag']}</span>";
                        $this->set_status( $statstring );
                        continue;
                     }
                  }
/*
                  // check for %replace:% fragment error
                  if( $optag == "%replace:%" ) {
                     $is_fragment = false;
                     if( false != $i = strpos( $target_file_contents, $good_target ) ) {
                        $k = $i-1;
                        while( $target_file_contents[$k] == " " || $target_file_contents[$k] == "\t" ) {
                           $k--;
                        }
                        if( $target_file_contents[$k] != "\n" ) {
                           $is_fragment = true;
                        }
                        else {
                           $k = $i + strlen( $good_target );
                           while( $target_file_contents[$k] == " " || $target_file_contents[$k] == "\t" ) {
                             $k++;
                           }
                           if( $k < strlen( $target_file_contents ) && $target_file_contents[$k] != "\r" ) {
                              $is_fragment = true;
                           }
                        }
                    if( $is_fragment ) {
                           $num_errors++;
                           $statstring .= "<span class=\"msgerror\">{$admtext['no_frag']}</span>";
                           $this->set_status( $statstring );
                           continue;
                        }
                     }
                  }
*/
                  $statstring .= "<span class=\"NINST\">{$admtext['notinst']}</span>";
                  $this->set_status( $statstring );
                  continue;
               } // locations
               /*************************************************************************
               FILE OPERATIONS
               *************************************************************************/
               elseif( $tags[$j]['name'] == 'copyfile' || $tags[$j]['name'] == 'copyfile2' ) {
                  $optional = '';
                  if( false !== strpos($tags[$j]['arg1'], "@" ) ) {
                     $optional = '@';
                     $tags[$j]['arg1'] = str_replace( "@", "", $tags[$j]['arg1'] );
                  }
                  else {
                     $this->num_required++;
                     $this->copies_required++;
                  }

                  $source_file = $tags[$j]['arg1'];

                  $statstring = "$line {$tags[$j]['line']}: <span class=\"tag\">%{$tags[$j]['name']}:</span><span class=\"tgtfile\">$optional$source_file</span>";
                  if( $tags[$j]['name'] == 'copyfile2' ) {
                     $statstring .= ":<span class=\"tgtfile\">{$tags[$j]['arg2']}</span>&nbsp;";
                  }
                  else {
                     $statstring .= "&nbsp;";
                  }
                  $source_path = $this->modspath.$source_file;
                  $filename = pathinfo( $source_path, PATHINFO_BASENAME );
                  $destination_path = $tags[$j]['name'] == 'copyfile'
                    ? $this->rootpath.$filename
                    : $this->rootpath.$tags[$j]['arg2'];

                  // warn if destination folder for copy op does not exist
                  $thefolder = pathinfo( $destination_path, PATHINFO_DIRNAME );
                  if( !file_exists( $thefolder ) ) {
                     if( !$optional ) {
                     $num_errors++;
                     $statstring .= "<span class=\"hilighterr msgbold\">{$admtext['nofolder']}</span>";
                     $this->set_status( $statstring );
                     }
                     continue;
                  }
                  if( file_exists( $destination_path ) ) {
                     if( !$optional ) {
                        // total number of elements
                        $this->num_installed++;
                        // total files copied
                        $this->copies_installed++;
                     }
                     $statstring .= $admtext['copied'];
                     $this->set_status( $statstring );
                     continue;
                  }
                  if( file_exists( $source_path ) ) {
                     if( $optional ) {
                        $statstring .= "({$admtext['optional']})&nbsp;";
                     }
                     $statstring .= "<span class=\"NCOPY\">{$admtext['notcopied']}</span>";
                     $this->set_status( $statstring );
                     continue;
                  }
                  else {
                     if( $optional ) {
                        $statstring .= "({$admtext['optional']})&nbsp;{$admtext['missing']}&nbsp;{admtext['willbypass']}";
                     }
                     else {
                        $num_errors++;
                        $statstring .= "<span class=\"hilighterr msgbold\">{$admtext['srcfilemissing']}</span>";
                     }
                     $this->set_status( $statstring );
                     continue;
                  }
               }
               // create new file
               elseif( $tags[$j]['name'] == 'newfile' ) {
                  $optional = '';
                  if( false !== strpos($tags[$j]['arg1'], "@" ) ) {
                     $optional = '@';
                     $tags[$j]['arg1'] = str_replace( "@", "", $tags[$j]['arg1'] );
                  }
                  else {
                     $this->num_required++;
                     $this->newfiles_required++;
                  }
                  $destination_file = $tags[$j]['arg1'];
                  $statstring = "$line {$tags[$j]['line']}: %<span class=\"tag\">{$tags[$j]['name']}:</span><span class=\"tgtfile\">$optional$destination_file</span>%&nbsp;";
                  $destination_path = $this->rootpath.$destination_file;

                  // warn if destination folder for new file does not exist
                  $thefolder = pathinfo( $destination_path, PATHINFO_DIRNAME );
                  if( !file_exists( $thefolder ) ) {
                     if( !$optional ) {
                     $num_errors++;
                     $statstring .= "<span class=\"hilighterr msgbold\">{$admtext['nofolder']}</span>";
                     $this->set_status( $statstring );
                     }
                     continue;
                  }

                  // new file installed check version number
                  if( file_exists( $destination_path ) ) {
                     if( !$optional ) {
                        $this->num_installed++;
                        $this->newfiles_installed++;
                     }
                     if( $tags[$j+1]['name'] == 'fileversion' ) {
                        // match cfg fileversion with installed newfile version
                        $fileversion = $tags[$j+1]['arg1'];
                        $newfile = file_get_contents( $destination_path );
                        if( false === strpos( $newfile, $fileversion ) ) {
//                           $num_errors++;
                           $this->num_installed--;
                           $statstring .= "<span class=\"hilighterr msgbold\">{$admtext['badversion']}</span>";
                           $this->set_status( $statstring );
                           continue;
                        }
                     }
                     $statstring .= $admtext['created'];
                     $this->set_status( $statstring );
                     continue;
                  }
                  // newfile does not exist
                  else {
                     $statstring .= "<span class=\"NCREATE\">{$admtext['nocreated']}</span>";
                     $this->set_status( $statstring );
                     continue;
                  }
                  if( !$optional ) {
                     $this->num_required--;
                     $this->newfiles_required--;
                  }
               }
            } // process parse table
         } // if not parese errors

         $id++;
         $errors = '';
         $warnings = '';
         $status = '';

         // These are parser errors
         if( count( $this->parse_errors ) ) {
            foreach( $this->parse_errors as $perrors ) {
               $num_errors++;
               $errors .= "$line {$perrors['line']}: %{$perrors['tag']} <span class=\"msgerror\">
               {$admtext[$perrors['text']]}</span><br />";
            }
            $status_header = self::CANTINST;
         }
         // these are syntax/op/environmental warnings
         else {
            if( $this->show_warnings ) {
               // start status with warnings
               if( count( $this->warnings ) ) {
                  $warnings .= "{$admtext['warnings']}:<br />";
               }
               foreach( $this->warnings as $warning ) {
                  $msgclass = 'msgnormal';
                  $warnings .= "$line {$warning['line']}: <span class=\"$msgclass\">
                  %{$warning['tag']} <strong>{$warning['info']}</strong>
                  {$admtext[$warning['text']]}</span><br />";
               }
               if( count( $this->warnings ) ) {
                  $warnings .= "<hr />";
               }
            }

            if( $this->num_installed > 0 && $this->num_installed < $this->num_required ) {
               $status_header = self::PARTINST;
            }
            elseif( !$num_errors ) {
               if( $this->num_required > 0 ) {
                  if( $this->num_installed == $this->num_required ) {
                     $status_header = self::INSTALLED;
                  }
                  elseif( $this->num_installed == 0 ) {
                     $status_header = self::OK2INST;
                  }
               }
            }
            else {
               $status_header = self::CANTINST;
            }
         }

         // output can be filtered here (now that status header is known)
         if( $this->filter == self::F_INSTALLED ) {
            if( $status_header != self::INSTALLED ) continue;
         }
         elseif( $this->filter == self::F_READY ) {
            if( $status_header != self::OK2INST ) continue;
         }
         elseif( $this->filter == self::F_CLEAN ) {
            if( $status_header != self::PARTINST ) continue;
         }
         elseif( $this->filter == self::F_BADCFG ) {
            if( $status_header != self::CANTINST ) continue;
         }

         $status = $warnings.$errors;

         // don't show status details for OK to Install and Installed
         //if( $status_header != self::OK2INST && $status_header != self::INSTALLED )
            $status .= $this->get_status();

         // modify styles per status type
         if( $status_header == self::PARTINST ) {
            $status = str_replace(
               array( "NINST", "NCOPY", "NCREATE" ),
               array( "hilight", "hilight", "hilight" ),
               $status
            );
         }
         else {
            $status = str_replace(
               array( "NINST", "NCOPY", "NCREATE" ),
               array( "none", "none", "none" ),
               $status
            );
         }

         // construct the folding status display
         $fstatus = $this->format_status( $status_header, $status, $num_errors, $id );

         // create affected files display
         $aff_files = $this->get_affected_files( $id );

/*************************************************************************
MAIN DISPLAY
*************************************************************************/

         $wiki = !empty( $this->wikipage ) ? $this->wikibase.$this->wikipage : '';

         $wikilink = "<div class=\"mminner center\">";
         if( !empty( $this->wikipage ) )
            $wikilink .= "<a href=\"$wiki\" target=\"_blank\"><img class=\"center\" src=\"classes/wiki16.png\"></a>";
         else
            $wikilink .= "&nbsp;";
         $wikilink .= "</div>";

         if( !$this->use_striping ) {
            $dbclass = 'databack';
         }
         elseif( $this->stripe_after > 0  ) {
            if( $dbx % $this->stripe_after == 0 ) {
               $dbclass = $dbclass == 'databack' ? "databackalt" : 'databack';
            }
         }
         $dbx++;

         echo "
   <tr class=\"modline\">";
   if ($this->filter != self::F_ALL) {
      echo "
		<td class=\"mmcell $dbclass\">
         <div class=\"mminner center checkpad\">
            <input class=\"sbox\" type=\"checkbox\" name=\"mods[$ix][selected]\" value=\"1\" />
            <input type=\"hidden\" name=\"mods[$ix][file]\" value=\"$this->cfgfile\" />
         </div>
      </td>";
      $ix++;
   }
   echo "
      <td id=\"flinka{$id}\" class=\"flink mmcell $dbclass\">
         <div class=\"mminner mmcellpad\">
            $id. <strong>$this->modname</strong>
         </div>
      </td>
      <td id=\"flinkb{$id}\" class=\"flink mmcell $dbclass\">
         <div class=\"mminner mmcellpad\">
            $this->cfgfile
         </div>
      </td>
      <td id=\"flinkc{$id}\" class=\"flink mmcell $dbclass center\">
         <div class=\"mminner mmcellpad\">";
   echo ltrim( $this->version, "vV" );
   echo "
         </div>
      </td>
      <td class=\"mmcell $dbclass center mmcellpad\">
         $wikilink
      </td>
      <td class=\"stcell\">
         $fstatus
      </td>
      <td class=\"afcell\">
         $aff_files
      </td>
   </tr>";
      } // each mod
   echo "

   </table>

   </td><!--tngshadow-->
   </tr><!--databack-->
   </table><!--mmlist-->

</form>";
   }  // list_mods

   /**********************************************************************
   FUNCTIONS
   **********************************************************************/
   protected function set_status( $string ){
      $this->status_string .= $string.'<br />';
   }

   protected function get_status() {
      return $this->status_string;
   }

   protected function init_status() {
      $this->status_string = '';
   }

   protected function format_status( $status_header, $status, $num_errors, $id ) {
      $admtext = $this->admtext;

      //$btn_install = "<button class=\"msgapproved\" type=\"submit\" name=\"submit\" value=\"install\" >{$admtext['install']}</button>";
      //$btn_install  = "<a href=\"?a=".self::INSTALL."&m=$this->cfgfile\"><button class=\"msgapproved\" type=\"button\">{$admtext['install']}</button></a>";
      $btn_install = "<button class=\"msgapproved\" type=\"button\" onclick='window.location.href=\"?a=".self::INSTALL."&m=$this->cfgfile\"'>{$admtext['install']}</button>";

      //$btn_delete = "<button class=\"msgerror\" type=\"submit\" onclick=\"return confirm( '{$this->admtext['confdelmod']}');\" name=\"submit\" value=\"delete\">{$this->admtext['delete']}</button>";
      //$btn_delete  = "<a href=\"?a=".self::DELETE."&m=$this->cfgfile\"><button class=\"msgerror\" type=\"button\" onclick=\"return confirm('{$this->admtext['confdelmod']}');\">{$this->admtext['delete']}</button></a>";
      $btn_delete = "<button class=\"msgerror\" type=\"button\" onclick=\"if(confirm('{$this->admtext['confdelmod']}')) {window.location.href='?a=".self::DELETE."&m=$this->cfgfile';}\">{$admtext['delete']}</button>";

      //$btn_remove = "<button type=\"submit\" name=\"submit\" value=\"remove\">{$this->admtext['uninstall']}</button>";
      //$btn_remove  = "<a href=\"?a=".self::REMOVE."&m=$this->cfgfile\"><button class=\"msgapproved\" type=\"button\">{$this->admtext['uninstall']}</button></a>";
      $btn_remove = "<button class=\"msgapproved\" type=\"button\" onclick='window.location.href=\"?a=".self::REMOVE."&m=$this->cfgfile\"'>{$admtext['uninstall']}</button>";

      //$btn_cleanup = "<button type=\"submit\" name=\"submit\" value=\"cleanup\">{$this->admtext['cleanup']}</button>";
      //$btn_cleanup  = "<a href=\"?a=".self::CLEANUP."&m=$this->cfgfile\"><button class=\"msgapproved\" type=\"button\">{$this->admtext['cleanup']}</button></a>";
      $btn_cleanup = "<button class=\"msgapproved\" type=\"button\" onclick='window.location.href=\"?a=".self::CLEANUP."&m=$this->cfgfile\"'>{$admtext['cleanup']}</button>";

      $btn_edit = '';
      if( $this->parameters ) {
         //$btn_edit = "<input button type=\"submit\" name=\"submit\" value=\"Edit Options\" />";
         //$btn_edit  = "<a href=\"admin_modeditor.php?a=".self::EDITP."&m=$this->cfgfile\"><button type=\"button\">{$admtext['edopts']}</button></a>";
         $btn_edit = "<button type=\"button\" onclick='window.location.href=\"admin_modeditor.php?a=".self::EDITP."&m=$this->cfgfile\"'>{$admtext['edopts']}</button>";
      }

      $options_link = '';

      $buttons = '';
      if( $status_header == self::OK2INST ) {
         $status_header = $admtext[$status_header];
         $style = "ready";
         $buttons = "<div>
                  $btn_install
                  $btn_delete
               </div>";
      }
      elseif( $status_header == self::CANTINST ) {
         $status_header = $admtext[$status_header];
         $style = "badcfg";
         $buttons = "<div>
                  $btn_delete
               </div>";
      }
      elseif( $status_header == self::INSTALLED ) {
         $status_header = $admtext[$status_header];
         if( !empty( $this->parameters ) ) {
			$status_option = $admtext['hasoptions'];
            $status_header .= " [$status_option]";
         }
         $style = "installed";
         $buttons = "<div>
                  $btn_remove
                  $btn_edit";
         if( $this->delete_installed ) {
            $buttons .=  "
               $btn_delete";
         }
         $buttons .= "</div>";
      }
      elseif( $status_header == self::PARTINST ) {
         $status_header = $admtext[$status_header];
         $style = "partinst";
         $buttons = "<div>
                  $btn_cleanup
                  $btn_delete
               </div>";
      }
      if( $this->num_required > 0 ) {
         $summary = "<hr />
               <ul class=\"results fieldnameback fieldname\">
                  <li>{$admtext['modsreq']}: $this->mods_required; {$admtext['modified']}: $this->mods_installed</li>
                  <li>{$admtext['copiesreq']}: $this->copies_required; {$admtext['copied']}: $this->copies_installed</li>
                  <li>{$admtext['newfilesreq']}: $this->newfiles_required; {$admtext['created']}: $this->newfiles_installed</li>
                  <li>{$admtext['errors']}: $num_errors</li>
               </ul>";
      }
      else
         $summary = '';


      return "<div class=\"$style\">
            <span class=\"modlink closed\" id=\"link{$id}\">
               $status_header
            </span>
         </div>
         <div class=\"moddiv $style\" id=\"link{$id}div\" style=\"display:none;padding:5px;\">
            $buttons
            <hr />
            $this->description
            <hr />
            $status
            $summary
         </div>";
   } // format status

   protected function save_affected_files( $tags ) {
      $this->afchange = array();
      $this->afcreate = array();
      $this->afcopy = array();
      $this->afcopy2 = array();
      for( $i=0;isset($tags[$i]); $i++ ) {
         if( $tags[$i]['name'] == 'target' )
            $this->afchange[] = $tags[$i]['arg1'];
         elseif( $tags[$i]['name'] == 'newfile' )
            $this->afcreate[] = $tags[$i]['arg1'];
         elseif( $tags[$i]['name'] == 'copyfile' )
            $this->afcopy[] = $tags[$i]['arg1'];
         elseif( $tags[$i]['name'] == 'copyfile2' )
            $this->afcopy2[] = $tags[$i]['arg2'];
      }
   }

   // return block of code containing afftected files per options
   protected function get_affected_files( $id ) {
      $admtext = $this->admtext;
      $retstr = "
         <div class=\"descpop1 nw imgcenter\" title=\"\">

            <img border=\"0\" src=\"img/tng_more.gif\" width=\"16\" alt=\"\" />

            <div>
               <table class=\"mmpopuptable\">
                  <tr>
                     <td class=\"normal fieldnameback fieldname mmpopuphdr\">
                        $id. <strong>$this->modname - </strong>$this->cfgfile
                     </td>
                  </tr>
               </table>";

      $filestr = '';
      $filestr .= $this->format_affrows( $this->afchange, "{$admtext['target']}" );
      $filestr .= $this->format_affrows( $this->afcreate, "{$admtext['newfile']}" );
      $filestr .= $this->format_affrows( $this->afcopy, "{$admtext['copiesfile']}" );
      $filestr .= $this->format_affrows( $this->afcopy2, "{$admtext['copiesfile2']}" );

      if( !empty( $filestr ) ) {
         $retstr .= "
               <table class=\"whiteback normal cellspace1 mmpad2\">
               $filestr
               </table>";
      }

      $retstr .= "
            </div>
         </div>"; // descpop
      return $retstr;
   } // get_affected_files

   private function format_affrows( $file_array, $label ) {
      $retstr = '';
      if( empty( $file_array ) ) return $retstr;
         $retstr .= "
                  <tr>
                     <td class=\"normal fieldnameback fieldname aligntop nw\">$label</td>
                     <td class=\"normal databack w100 mmpadleft\">";
      foreach( $file_array as $listing ) {
         if( $listing == 'files' ) continue;
         $retstr .= "
                        $listing<br />";
      }
      $retstr .= "
                     </td>
                  </tr>";
      return $retstr;
   }

/* --------- block 2 removed ---------
   protected function list_aff_files( $regex, $contents ) {
      $files = array();
      $array = preg_split( $regex, $contents );
      for( $i=1; isset( $array[$i] ); $i++ ) {
         $files[] = preg_replace( "#^([^%]*)%.*$#s", '${1}', $array[$i] );
      }
      return $files;
   }

-------------block 2 ------- */

} // modlister class
?>

