<?php
/*
   Gen III Mod Manager parser.

   This class parses and stores mod config file directives, data and error information
   in a table for use by other classes.

   Methods:
      protected function get_modfile_names();
      protected function get_modlist();
      public function parse( $cfgpath );
      protected function fix_eol( $instr );
      protected function merge_tag_lines( $lines, $idx );
      protected function merge_code_lines( $lines, $idx, $terminator);
      protected function merge_lines( $lines, $num_lines, $start_line_index, $terminator)
      public function show_parse_tables();
      private function show_table ( $table )
*/
//140825 with tag 'parameter' place all after the 1st colon in arg1

include_once 'classes/modbase.class.php';

class modparser extends modbase {
   const NOERR       = 0;
   const ERROR      = -1;
   const WARNING     = -2;

   // indexes into language text arrays

   //const BADTARGET   = 'badtarget';
   const BADVERSION  = 'badversion';
   const BOMFOUND    = 'bomfound';
   const CANTINST    = 'cantinst';
   const EMPTYFILE   = 'emptyfile';
   //const ERRORS      = 'errors';
   //const EXCUSED     = 'excused';
   const FORMAT      = 'format';
   //const INSTALLED   = 'installed';
   const LINE        = 'line';
   const MISSING     = 'missing';
   //const NOACCESS    = 'noaccess';
   const NOACT       = 'noact';
   const NOCODE      = 'nocode';
   const NOEND       = 'noend';
   const NOCFGFILE   = 'nocfgfile';
   //const NOLOCATION  = 'nolocation';
   const NOCOMPS     = 'nocomps';
   //const NOMSG       = 'nomsg';
   const NOTARGET    = 'notarget';
   //const NOTGTFILE   = 'notgtfile';
   //const NOTINST     = 'notinst';
   //const NOTUNIQUE   = 'notunique';
   //const OK2INST     = 'ok2inst';
   //const OKAY        = 'okay';
   //const PARAMETER   = 'parameter';
   //const PARTINST    = 'partinst';
   const REQTAG      = 'reqtag';
   //const REQUIRED    = 'required';
   const SECNOTERM   = 'secnoterm';
   const TAGNOTERM   = 'tagnoterm';
   const TAGUNK      = 'tagunk';
   const UNXEND      = 'unxend';
   const WHITESPACE  = 'whitespace';

   public $tags = array();
   public $parse_errors = array();
   public $warnings = array();

   protected $tagdefs = array(
      'code'         => array( 'args' => '1', 'multi' => '1' ),
      'copyfile'     => array( 'args' => '1', 'multi' => '0' ),
      'copyfile2'    => array( 'args' => '2', 'multi' => '0' ),
      'desc'         => array( 'args' => '1', 'multi' => '1' ),
      'description'  => array( 'args' => '1', 'multi' => '1' ),
      'end'          => array( 'args' => '0', 'multi' => '0' ),
      'fileend'      => array( 'args' => '0', 'multi' => '0' ),
      'fileoptional' => array( 'args' => '0', 'multi' => '0' ),
      'fileversion'  => array( 'args' => '1', 'multi' => '0' ),
      'insert'       => array( 'args' => '1', 'multi' => '0' ),
      'location'     => array( 'args' => '0', 'multi' => '0' ),
      'name'         => array( 'args' => '1', 'multi' => '0' ),
      'newfile'      => array( 'args' => '1', 'multi' => '0' ),
      'parameter'    => array( 'args' => '2', 'multi' => '0' ),
      'replace'      => array( 'args' => '0', 'multi' => '0' ),
      'target'       => array( 'args' => '1', 'multi' => '0' ),
      'triminsert'   => array( 'args' => '1', 'multi' => '0' ),
      'trimreplace'  => array( 'args' => '0', 'multi' => '0' ),
      'version'      => array( 'args' => '1', 'multi' => '0' ),
      'wikipage'     => array( 'args' => '1', 'multi' => '0' )
   );

   protected $optag_args = array (
      'insert' => array( 'before', 'after' ),
      'triminsert' => array ( 'before', 'after' ),
   );
   protected $op_tags = array(
      '%insert:before%',
      '%triminsert:before%',
      '%insert:after%',
      '%triminsert:after%',
      '%replace:%',
      '%trimreplace:%'
   );
   protected $configs = array(
      'config.php',
      'customconfig.php',
      'importconfig.php',
      'logconfig.php',
      'mapconfig.php',
      'pedconfig.php',
      'templateconfig.php',
      'mmconfig.php'
   );
   protected $tag_regex = "#^\s*%([^:]+):?([^:%]*):?([^%]*)%?#";

   // parameter is exception -- it may have 0-inifinte number of parameters
   protected $param_regex = "#^\s*%(parameter):([^:]+):([^%]*)%?$#";
   /*********************************************************************/

	function __construct( $objinits ) {
      // pass arguments thru to base class
      parent::__construct( $objinits );
	}

   public function parse( $cfgpath ) {
      // (re)initialize data
      $this->cfgpath = $cfgpath;
      $this->modname = '';
      $this->version = '';
      $this->description = '';
      $this->wikipage = '';
      $this->tags = array();
      $this->parse_errors = array();
      $this->warnings = array();
      $pidx = 0;  // errors table index
      $widx = 0;  // warnings table index

      if( !file_exists( $cfgpath ) ) {
         $this->parse_errors[$pidx]['line'] = 0;
         $this->parse_errors[$pidx]['tag'] = '';
         $this->parse_errors[$pidx]['info'] = $cfgpath;
         $this->parse_errors[$pidx++]['text'] = self::NOCFGFILE;
         return;
      }
      // get contents of mod config file and normalize line endings
      $config_contents = $this->fix_eol( file_get_contents( $cfgpath ) );
      if( empty( $config_contents ) ) {
         $this->parse_errors[$pidx]['line'] = 0;
         $this->parse_errors[$pidx]['tag'] = '';
         $this->parse_errors[$pidx]['info'] = $cfgpath;
         $this->parse_errors[$pidx++]['text'] = self::EMPTYFILE;
         return;
      }


      $lines = preg_split( "#\r\n#s", $config_contents );
      $num_lines = count( $lines );
      unset( $config_contents );

      // remove utf8 bom (regex won't work with it) and set a warning
      if( preg_match( "#^\xEF\xBB\xBF+.*$#", $lines[0], $matches) ) {
         $lines[0] = preg_replace( "#\xEF\xBB\xBF#", "", $lines[0] );
         $this->warnings[$widx]['line'] = 0;
         $this->warnings[$widx]['tag'] = '';
         $this->warnings[$widx]['info'] = $this->cfgpath;
         $this->warnings[$widx++]['text'] = self::BOMFOUND;
      }

      /*************************************************************************
      BUILD TAGS TABLE FROM CONFIG FILE
      *************************************************************************/
      $num_targets = 0;

      $j = 0; // tags array index
      for( $i=0; isset( $lines[$i] ); $i++ ) {

         // %parameter requires special handling because arg2 can be multiline
         // with unlimited number of arguments
         if( !preg_match( $this->param_regex, $lines[$i], $matches ) ) {
            if( !preg_match( $this->tag_regex, $lines[$i], $matches ) )
               continue;
          }

          // process 'parameter' -- allows multiline variable values
          else {
             $args = $this->merge_tag_lines( $lines, $i );
             //echo __FILE__,' ',__LINE__,' ', $args,'<br />';
             $a = preg_split( "#:#", $args, 2 );
             $matches[2] = $a[0];
             $matches[3] = $a[1];
             //echo __FILE__,' ',__LINE__,'<br />';print_r( $matches );exit;
          }
         // remove whitespace from name
         $tagname = trim( str_replace( " ", "", $matches[1] ) );

         // Tag name must be valid
         if( !isset( $this->tagdefs[$tagname] ) ) {
            $this->parse_errors[$pidx]['line'] = $i+1; // line number is one greater than line index
            $this->parse_errors[$pidx]['tag'] = $matches[1];
            $this->parse_errors[$pidx]['info'] = '';
            $this->parse_errors[$pidx++]['text'] = self::TAGUNK;
            return;
         }

         // Description directly to class data -- no table entry
         if( $tagname == 'description' ) {
            $description = $this->merge_tag_lines( $lines, $i );
            if( $description == self::ERROR ) {
               $this->parse_errors[$pidx]['line'] = $i;
               $this->parse_errors[$pidx]['tag'] = 'description';
               $this->parse_errors[$pidx]['info'] = '';
               $this->parse_errors[$pidx++]['text'] = self::TAGNOTERM;
               return;
            }
            else {
               $this->description = $description;
               continue;
            }
         }

         // special handling for desc because it may span several lines and contain colons (:);
         // special handling for multiline tag because it may span several lines and contain colons (field mm separators);
         if( $this->tagdefs[$tagname]['multi'] ) {
            $arg1 = $this->merge_tag_lines( $lines, $i );
            if( $arg1 == self::ERROR ) {
               $this->parse_errors[$pidx]['line'] = $i+1;
               $this->parse_errors[$pidx]['tag'] = $tagname;
               $this->parse_errors[$pidx]['info'] = '';
               $this->parse_errors[$pidx++]['text'] = self::TAGNOTERM;
            }
            $matches[2] = $arg1;
         }

         // get parameter default value from desc parentheses
         if($tagname == 'desc' ) {
            $matches[3] = preg_replace( "#^[^%\(]*\(([^\)]*)\).*$#s", '${1}', $matches[2] );
         }


         // SINGLE LINE TAGS
         if( !$this->tagdefs[$tagname]['multi'] && $tagname != 'parameter' ) {

            //special handling for %name:%
            if( $tagname == 'name' ) {
               if( preg_match( "#%name:\s#", $matches[0] ) ) {
                  $temp = preg_replace( "#(%name:)\s#", '${1}', $matches[0] );
                  preg_match( $this->tag_regex, $temp, $matches );
                  $this->warnings[$widx]['line'] = $i+1;
                  $this->warnings[$widx]['tag'] = $tagname;
                  $this->warnings[$widx]['info'] = '';
                  $this->warnings[$widx++]['text'] = self::WHITESPACE;
               }
               if( $this->compress_names ) {
                  $temp = preg_replace( "#\s#", "", $matches[0] );
                  preg_match( $this->tag_regex, $temp, $matches );
               }
            }
            elseif( preg_match( "#\s#", $matches[0] ) ) {
               $temp = preg_replace( "#\s#", "", $matches[0] );
               preg_match( $this->tag_regex, $temp, $matches );
               $this->warnings[$widx]['line'] = $i+1;
               $this->warnings[$widx]['tag'] = $tagname;
               $this->warnings[$widx]['info'] = '';
               $this->warnings[$widx++]['text'] = self::WHITESPACE;
            }

            // add terminator if missing
            if( !preg_match( "#%[^%]+%#", $matches[0] ) ) {
               $matches[0].= "%";
               $this->warnings[$widx]['line'] = $i+1;
               $this->warnings[$widx]['tag'] = $tagname;
               $this->warnings[$widx]['info'] = '';
               $this->warnings[$widx++]['text'] = self::TAGNOTERM;
            }

            // Header tag values directly to class data, no table entries
            if( $tagname == 'name' ) {
               $this->modname = $matches[2];
               continue;
            }
            elseif ( $tagname == 'version' ) {
               $this->version = $matches[2];
               continue;
            }
            elseif ( $tagname == 'wikipage' ) {
               $this->wikipage = $matches[2];
               continue;
            }

            // Tag Format errors
            $args = $this->tagdefs[$matches[1]]['args'];

            // Parameter tag is allowed to have empty second argument
            if( $matches[1] == 'parameter' ) {
               if( !preg_match( "#^\s*%parameter:[^:]+:[^%]*%#", $matches[0] ) ) {
                  $this->parse_errors[$pidx]['line'] = $i+1;
                  $this->parse_errors[$pidx]['tag'] = $matches[1];
                  $this->parse_errors[$pidx]['info'] = '';
                  $this->parse_errors[$pidx++]['text'] = self::FORMAT;
                  return;
               }
            }
            elseif( $args == 0 && !preg_match( "#%[^:]+:%#", $matches[0] ) ) {
               $this->parse_errors[$pidx]['line'] = $i+1;
               $this->parse_errors[$pidx]['tag'] = $matches[1];
               $this->parse_errors[$pidx]['info'] = '';
               $this->parse_errors[$pidx++]['text'] = self::FORMAT;
               return;
            }
            // Formats errors -- one arg
            elseif( $args == 1 && !preg_match( "#%[^:]+:[^%]+%#", $matches[0] ) ) {
               $this->parse_errors[$pidx]['line'] = $i+1;
               $this->parse_errors[$pidx]['tag'] = $matches[1];
               $this->parse_errors[$pidx]['info'] = '';
               $this->parse_errors[$pidx++]['text'] = self::FORMAT;
               return;
            }
            // Format errors -- two args
            elseif( $args == 2 && !preg_match( "#%[^:]+:[^:]+:[^%]+%#", $matches[0] ) ) {
               $this->parse_errors[$pidx]['line'] = $i+1;
               $this->parse_errors[$pidx]['tag'] = $matches[1];
               $this->parse_errors[$pidx]['info'] = '';
               $this->parse_errors[$pidx++]['text'] = self::FORMAT;
               return;
            }

         }

         // Add new record to tags table
         $this->tags[$j]['line'] = $i+1;  // line number is one greater than line index
         $this->tags[$j]['raw'] = $matches[0];
         $this->tags[$j]['name'] = $matches[1];
         $this->tags[$j]['arg1'] = isset( $matches[2] ) ? $matches[2] : '';
         $this->tags[$j]['arg2'] = isset( $matches[3] ) ? $matches[3] : '';
         $this->tags[$j]['info'] = '';
         $this->tags[$j]['state'] = '';
         $this->tags[$j]['text'] = '';

         if( $this->tags[$j]['name'] == 'target' ) $num_targets++;
         if( $this->tags[$j]['name'] == 'target' || $this->tags[$j]['name'] == "newfile" ) {
            if( preg_match( "#\\$([^/]+)(.+)#", $this->tags[$j]['arg1'], $pathparts ) ) {
               if( !empty( $pathparts[1] ) ) {
                  $this->tags[$j]['arg1'] = $this->{$pathparts[1]}.$pathparts[2];
               }
            }
         }
         if( $this->tags[$j]['name'] == 'copyfile2' ) {
            if( preg_match( "#\\$([^/]+)(.+)#", $this->tags[$j]['arg2'], $pathparts ) ) {
               if( !empty( $pathparts[1] ) ) {
                  $this->tags[$j]['arg2'] = $this->{$pathparts[1]}.$pathparts[2];
               }
            }
         }

         // Add code block to table as for %location: or optag
         if( $matches[1] == 'location' || in_array( $matches[0], $this->op_tags ) ) {
            $mmcode = $this->merge_code_lines( $lines, $i, "%end:%" );
            if( $mmcode == self::ERROR ) {
               // termination error
               $this->parse_errors[$pidx]['line'] = $i+1;
               $this->parse_errors[$pidx]['tag'] = $matches[1];
               $this->parse_errors[$pidx]['info'] = '';
               $this->parse_errors[$pidx++]['text'] = self::SECNOTERM;
               return;
            }
            elseif( empty( $mmcode ) ) {
               // no code error
               $this->parse_errors[$pidx]['line'] = $i+1;
               $this->parse_errors[$pidx]['tag'] = $matches[1];
               $this->parse_errors[$pidx]['info'] = '';
               $this->parse_errors[$pidx++]['text'] = self::NOCODE;
               return;
            }
            else {
               $j++; // create next tag for %:code:%
               $this->tags[$j]['line'] = $i+1; // line nr is always one greate than line index
               $this->tags[$j]['raw'] = "%:code:%";
               $this->tags[$j]['name'] = "code";
               $this->tags[$j]['arg1'] = $mmcode;
               $this->tags[$j]['arg2'] = '';
               $this->tags[$j]['info'] = '';
               $this->tags[$j]['state'] = '';
               $this->tags[$j]['text'] = '';
            }
         }

         // Add code block to table for %fileverson:
         if( $matches[1] == 'fileversion' ) {
            $mmcode = trim( $this->merge_code_lines( $lines, $i, "%fileend:%" ) );

            if( $mmcode == self::ERROR ) {
               // termination error
               $this->parse_errors[$pidx]['line'] = $i+1;
               $this->parse_errors[$pidx]['tag'] = $matches[1];
               $this->parse_errors[$pidx]['info'] = '';
               $this->parse_errors[$pidx++]['text'] = self::SECNOTERM;
               return;
            }
            elseif( empty( $mmcode ) ) {
               // no code error
               $this->parse_errors[$pidx]['line'] = $i+1;
               $this->parse_errors[$pidx]['tag'] = $matches[1];
               $this->parse_errors[$pidx]['info'] = '';
               $this->parse_errors[$pidx++]['text'] = self::NOCODE;
               return;
            }
            else {
               $j++; // create next tag for %:code:%
               $this->tags[$j]['line'] = $i+1; // line nr is always one greate than line index
               $this->tags[$j]['raw'] = "%:code:%";
               $this->tags[$j]['name'] = "code";
               $this->tags[$j]['arg1'] = $mmcode;
               $this->tags[$j]['arg2'] = '';
               $this->tags[$j]['info'] = '';
               $this->tags[$j]['state'] = '';
               $this->tags[$j]['text'] = '';
            }
         }
         $j++;
      } // pass 1 -- build tags table
      //echo __FILE__,' ',__LINE__,'<br />';print_r( $this->tags );exit;
      /*************************************************************************
      CHECK PARSING FOR MISSING ELEMENTS AND OTHER ERRORS
      *************************************************************************/
      $missingtag = false;
      $errstr = '';
      if( empty( $this->modname ) ) $errstr .= 'name';
      elseif( empty( $this->version ) ) $errstr .= 'verson';
      elseif( empty( $this->description ) ) $errstr .= 'description';
      elseif( empty( $num_targets ) ) $errstr .= 'target';
      if( !empty( $errstr ) ) {
         $this->parse_errors[$pidx]['line'] = 0;
         $this->parse_errors[$pidx]['tag'] = $errstr;
         $this->parse_errors[$pidx]['info'] = '';
         $this->parse_errors[$pidx++]['text'] = self::REQTAG;
         return;
      }

      $op_tags = $this->op_tags;
      $tags = $this->tags;
      $num_tags = count( $this->tags );
      $eof_error = false;
      for( $j=0; $j<$num_tags; $j++ ) {

         // Missing Location or Parameter Error
         if( $this->tags[$j]['name'] == 'target' && $this->tags[$j]['arg1'] != 'files' ) {
            //search table for 'location' or 'target' whichever comes first
            $found = false;
            for( $i=$j+1; $i<$num_tags; $i++ ) {
               if( $this->tags[$i]['name'] == 'target' ) {
                  break;
               }
               if( $this->tags[$i]['name'] == 'location' ) {
                  $found = true;
                  break;
               }
               if( $this->tags[$i]['name'] == 'parameter' ) {
                  $found = true;
                  break;
               }
            }
            if( !$found ) {
               $this->parse_errors[$pidx]['line'] = $this->tags[$j]['line'];
               $this->parse_errors[$pidx]['tag'] = $this->tags[$j]['name'];
               $this->parse_errors[$pidx]['info'] = '';
               $this->parse_errors[$pidx++]['text'] = self::NOCOMPS;
               return;
            }
         }

         // check for op directive with missing location
         if( in_array( $this->tags[$j]['raw'], $this->op_tags ) ) {
            if( $this->tags[$j-3]['name'] != 'location' ) {
               $this->parse_errors[$pidx]['line'] = $this->tags[$j]['line'];
               $this->parse_errors[$pidx]['tag'] = $this->tags[$j]['name'];
               $this->parse_errors[$pidx]['info'] = '';
               $this->parse_errors[$pidx++]['text'] = self::NOTARGET;
               return;
            }
         }

         // Missing code, Op directives or end tags in location
         if( $this->tags[$j]['name'] == 'location' ) {
            $j++;
            if( $j < $num_tags ) {
               // No code section error
               if( $this->tags[$j]['name'] == 'code' ) {
                  $code = trim( $this->tags[$j]['arg1'] );
                  if( empty( $code ) ) {
                     $this->parse_errors[$pidx]['line'] = $this->tags[$j]['line'];
                     $this->parse_errors[$pidx]['tag'] = $this->tags[$j]['name'];
                     $this->parse_errors[$pidx]['info'] = '';
                     $this->parse_errors[$pidx++]['text'] = self::NOCODE;
                     return;
                  }
               }
            }
            else {
               $eof_error = true;
               break;
            }
            $j++;
            if( $j < $num_tags ) {
               // Code section not terminated with %end:% tag
               if( $this->tags[$j]['raw'] != '%end:%' ) {
                  $this->parse_errors[$pidx]['line'] = $this->tags[$j]['line'];
                  $this->parse_errors[$pidx]['tag'] = $this->tags[$j]['name'];
                  $this->parse_errors[$pidx]['info'] = '';
                  $this->parse_errors[$pidx++]['text'] = self::NOEND;
                  return;
               }
            }
            else {
               $eof_error = true;
               break;
            }
            $j++;
            if( $j < $num_tags ) {
               // No operational tag
               if( !in_array( $this->tags[$j]['raw'], $op_tags ) ) {
                  $this->parse_errors[$pidx]['line'] = $this->tags[$j]['line'];
                  $this->parse_errors[$pidx]['tag'] = $this->tags[$j]['name'];
                  $this->parse_errors[$pidx]['info'] = '';
                  $this->parse_errors[$pidx++]['text'] = self::NOACT;
                  return;
               }
            }
            else {
               $eof_error = true;
               break;
            }
            $j++;
            if( $j < $num_tags ) {
               // No insert/replace code
               if( $this->tags[$j]['name'] == 'code' ) {
                  $code = trim( $this->tags[$j]['arg1'] );
                  if( empty( $code ) ) {
                     $this->parse_errors[$pidx]['line'] = $this->tags[$j]['line'];
                     $this->parse_errors[$pidx]['tag'] = $this->tags[$j]['name'];
                     $this->parse_errors[$pidx]['info'] = '';
                     $this->parse_errors[$pidx++]['text'] = self::NOCODE;
                  }
               }
            }
            else {
               $eof_error = true;
               break;
            }
            $j++;
            if( $j < $num_tags ) {
               // Code section not terminated
               if( $this->tags[$j]['raw'] != '%end:%' ) {
                  $this->parse_errors[$pidx]['line'] = $this->tags[$j]['line'];
                  $this->parse_errors[$pidx]['tag'] = $this->tags[$j]['name'];
                  $this->parse_errors[$pidx]['info'] = '';
                  $this->parse_errors[$pidx++]['text'] = self::NOEND;
               }
            }
            else {
               $eof_error = true;
               break;
            }
            $j++;
         } // location elements

         // check for missing fileversion and fileend tags
         if( $j < $num_tags && $this->tags[$j]['name'] == 'newfile' ) {
            $j++;
            if( $this->tags[$j]['name'] == 'fileoptional' ) $j++;
            if( $j < $num_tags ) {
               if( $this->tags[$j]['name'] != 'fileversion' ) {
                  $this->parse_errors[$pidx]['line'] = $this->tags[$j]['line'];
                  $this->parse_errors[$pidx]['tag'] = $this->tags[$j]['name'];
                  $this->parse_errors[$pidx]['info'] = '';
                  $this->parse_errors[$pidx++]['text'] = self::MISSING;
               }
               // Check fileversion against versiion
               else {
                  $fileversion = $this->tags[$j]['arg1'];
                  $j++;
                  if( $j < $num_tags ) {
                     if( $this->tags[$j]['name'] != 'code' ) {
                        $this->parse_errors[$pidx]['line'] = $this->tags[$j]['line'];
                        $this->parse_errors[$pidx]['tag'] = $this->tags[$j]['name'];
                        $this->parse_errors[$pidx]['info'] = '';
                        $this->parse_errors[$pidx++]['text'] = self::NOCODE;
                     }
                     else {
                        preg_match( "#%version:(.*)%#", $this->tags[$j]['arg1'], $matches );
                        if( !isset( $matches[1] ) || $fileversion != $matches[1] ) {
                           $this->parse_errors[$pidx]['line'] = $this->tags[$j]['line'];
                           $this->parse_errors[$pidx]['tag'] = $this->tags[$j]['name'];
                           $this->parse_errors[$pidx]['info'] = '';
                           $this->parse_errors[$pidx++]['text'] = self::BADVERSION;
                        }
                     }
                  }
                  else {
                     $eof_error = true;
                     break;
                  }
               }
            }
            else {
               $eof_error = true;
               break;
            }
            $j++;
            if( $j < $num_tags ) {
               if( $this->tags[$j]['name'] != 'fileend' ) {
                  $this->parse_errors[$pidx]['line'] = $this->tags[$j]['line'];
                  $this->parse_errors[$pidx]['tag'] = $this->tags[$j]['name'];
                  $this->parse_errors[$pidx]['info'] = '';
                  $this->parse_errors[$pidx++]['text'] = self::MISSING;
               }
            }
            else {
               $eof_error = true;
               break;
            }
            $j-2;
         } // newfile check
      } // check table for missing elements

      if( $eof_error ) {
         $this->parse_errors[$pidx]['line'] = $num_lines - 1; // line number is one greater that line index
         $this->parse_errors[$pidx]['tag'] = '';
         $this->parse_errors[$pidx]['info'] = $cfgpath;
         $this->parse_errors[$pidx++]['text'] = self::UNXEND;
      }
      //print_r( $this->tags );exit;
      if( count( $this->parse_errors ) )
         return;
   } // parse function

/*************************************************************************
   FUNCTIONS
*************************************************************************/
   protected function get_modfile_names() {
      $fileNames = array();

      if( $handle = opendir($this->modspath ) ) {
         while( false !== ( $file = readdir( $handle ) ) ) {
            if( pathinfo( $file, PATHINFO_EXTENSION ) == 'cfg' ) {
               $fileNames[] = $file;
            }
         }
         closedir( $handle );
         sort( $fileNames );
      }
      return $fileNames;
   }

   // returns file names (no path) as array indexes and mod name as array values
   protected function get_modlist() {
      $mod_list = array();

      if( !is_dir( $this->modspath ) ) {
         return self::NOPATH;
      }
      // get file list -- look up names & add to list
      $modfiles = $this->get_modfile_names();
      foreach( $modfiles as $modfile ) {
         $contents = file_get_contents( $this->modspath.$modfile );
         preg_match('#.*%name:([^%]*).*#s', $contents, $matches);
         if( isset( $matches[1] ) )
            $mod_list[$modfile] = trim( $matches[1] );
         else
            $mod_list[$modfile] = '';
      }
      return $mod_list;
   }
   // merge multi line argument until the terminator (%) is reached,
   // or another tag or end of file occurs
   // CRLF in multiline tags only exist to make the cfg readable.
   // They have no meaning for presentation (use <br />)
   // returns everything after first colon up to %
   protected function merge_tag_lines( $lines, $idx ) {

      // is tag on one line and terminated with a '%'?
      if( preg_match( "#[^%]*%[^%]*%#", $lines[$idx] ) ) {
         return preg_replace( "#[^:]*:([^%]*)%#", '${1}', $lines[$idx] );
      }
      else{
         $new_lines = preg_replace( "#[^:]*:(.*)$#", '${1}', $lines[$idx] );
         for( $i=$idx+1; isset( $lines[$i] ); $i++ ) {
            $num_terms = substr_count( $lines[$i], "%" );

            // must have found another tag
            if( $num_terms > 1 ) {
               return self::ERROR;
            }
            elseif( $num_terms == 1 ) {
               $new_lines .= "\r\n".preg_replace( "#([^%]*)%#", '${1}', $lines[$i] );
               return trim( $new_lines, "\r\n" );
            }
            // merge new line
            $new_lines .= "\r\n".$lines[$i];
         } // loop through lines

         // all line exhausted, must have reached the end of the file
         return self::ERROR;
      }
   }

   // merge multi line argument until terminator is reached, or another tag or end of file
   protected function merge_code_lines( $lines, $idx, $terminator) {
      $new_lines = '';

      for( $i=$idx+1; isset( $lines[$i] ); $i++ ) {

         // find terminator?
         if( false !== strpos( $lines[$i], $terminator ) ) {
            return preg_replace( "#\r\n$#", '', $new_lines );
         }

         // run into another tag?
         if( preg_match( "#^\s*%[^:]+:[^%]*%$#", $lines[$i] ) ) {
            if( $terminator != "%fileend:%" || false === strpos( $lines[$i], "%version" ) ) {
               return self::ERROR;
            }
         }
         $new_lines .= $lines[$i] . "\r\n";
      }
      // all lines exhausted without finding the terminator
      return self::ERROR;
   }

      // merge lines until terminator is reached, or another tag or end of file
   protected function merge_lines( $lines, $num_lines, $start_line_index, $terminator) {

      $ret_line = '';
      $terminator_found = false;
      for( $i=$start_line_index; $i < $num_lines; $i++ ) {
         // does it contain a terminator?
         if( substr_count( $lines[$i], $terminator ) == 1 ) {
            $terminator_found = true;
            if( $terminator == "%" ) {
               $ret_line .= $lines[$i]; // without CRLF
               $ret_line = str_replace( "%", "", $ret_line );
               return $ret_line;
            }
            else break;
         }
         elseif( preg_match( $this->tag_regex, $lines[$i] ) ) {
            break;
         }
         // merge new line
         $ret_line .= $lines[$i] . "\r\n";
      } // loop through lines

      if( !$terminator_found ) {
         return self::ERROR;
      }

      // remove final CRLF -- original blank lines will remain
      $ret_line = preg_replace( "#\r\n$#", '', $ret_line );
      return $ret_line;
   }
   // for developers
   public function show_parse_tables() {

      echo "<p><strong>name:", $this->modname,"</strong></p>";
      echo "<p><strong>version:", $this->version,"</strong></p>";
      echo "<p><strong>wikipage:", $this->wikipage,"</strong></p>";
      echo "<p><strong>description:", $this->description,"</strong></p>";
      echo "<hr />";
      if( !empty( $this->warnings ) ) {
         echo "<h3>Warnings:</h3>",$this->show_table( $this->warnings ), "<br />";
         echo "<hr />";
      }
      if( !empty( $this->parse_errors ) ) {
      echo "<h3>Errors:</h3>",$this->show_table( $this->parse_errors ), "<br />";
      echo "<hr />";
      }
      if( !empty( $this->tags ) )
      echo "<h3>Tags:</h3>",$this->show_table( $this->tags ), "<br />";
   }

   private function show_table ( $table ) {
      $outstr = '';

      if( !empty( $table ) ) {

         // create a table header
         $outstr = "
   <table class=\"normal w100\">
      <tr>";
         foreach( $table[0] as $key => $value ) {
            $outstr .= "
         <th>$key</th>";
         }
         $outstr .= "
      </tr>";

         // add rows to table
         foreach( $table as $row ) {
            $outstr .= "
      <tr>";
            foreach( $row as $key => $value ) {
              $outstr .= "
         <td class=\"grayborder\">
            <div>
               ".htmlentities( $value )."
            </div>
         </td>";
            }
            $outstr .= "
      </tr>";
         }
         $outstr .= "
   </table>";
      }
      return $outstr;
   }
} // modparser class
?>
