<?php

/* Gen III Modhandler Remover Class

   Methods:
      public function remove( $cfgpath );
      public function batch_remove( $cfgpathlist );

   141029 -- simplify target processing (%location)
*/

include_once 'modparser.class.php';

class modremover extends modparser {

   public $classID = "remover";

   public function __construct( $objinits ) {
      // pass arguments to base class
      parent::__construct( $objinits );
   }

   public function remove( $cfgpath ) {
      //echo __FILE__,' ',__LINE__," $cfgpath";exit;
      $this->cfgpath = $cfgpath;
      $this->cfgfile = $cfgfile = pathinfo( $cfgpath, PATHINFO_BASENAME );

      $this->mod_status = '';
      $admtext = $this->admtext;

      $this->parse( $cfgpath );

      $tags = $this->tags;
      $num_tags = count( $tags );

		$locations_installed = 0;
      $locations_removed = 0;

      $files_installed = 0;
      $files_removed = 0;

      $num_errors = 0;

      $line = $this->admtext['line'];


      // start logging this removal
      $this->new_logevent( "{$admtext['removing']} <strong>$cfgpath</strong>" );

      // if there were fatal parsing errors, log them and exit to the log
      if( count( $this->parse_errors ) ) {
         $this->modname = $cfgfile;
         $this->mod_status = self::ERRORS;
         $this->add_logevent( "{$admtext['removing']} <strong>$cfgpath</strong>" );
         foreach( $this->parse_errors as $parse_err ) {
            $idx = $parse_err['text'];
            $this->add_logevent("<span class=\"msgerror\">$cfgfile</span> <span class=\"hilighterr msgbold\">{$admtext[$idx]}</span>");
         }
         $this->write_eventlog();
         return false;
      }

      // tags processing loop
      for( $j=0; $j<$num_tags; $j++ ) {

/*************************************************************************
         TARGETS
*************************************************************************/
         if( $tags[$j]['name'] == 'target' ) {
            // skip "files" section here -- copyfiles done below
            if( $tags[$j]['arg1'] == 'files') continue;
            // Flush current target file buffer;
            if( !empty( $active_target_file ) && !empty( $target_file_contents ) ) {
               file_put_contents( $active_target_file, $target_file_contents );
            }
            $fileoptional = false;
            $target_file_basename = '';
            $active_target_file = '';
            $target_file_contents = '';

            // get file path info
            $path = in_array( $tags[$j]['arg1'], $this->configs ) ? $this->subroot : $this->rootpath;
            $target_file_basename = $tags[$j]['arg1'];
            $active_target_file = $path.$tags[$j]['arg1'];

            $fileoptional = $tags[$j+1]['name'] == 'fileoptional' ? true : false;
            $logstring = "$line {$tags[$j]['line']}: <span class=\"tag\">%target:</span><span class=\"tgtfile\">$target_file_basename</span><span class=\"tag\">%</span>&nbsp;";

            if( !file_exists( $active_target_file ) ) {
               if( $fileoptional ) {
                  $logstring .= "{$admtext['optmissing']}&nbsp;<span class=\"msgapproved\">{$admtext['bypassed']}</span>";
                  $this->add_logevent( $logstring );
               }
               else {
                  $num_errors++;
                  $logstring .= "<span class=\"msgerror\">{$admtext['tgtmissing']}</span>";
                  $this->add_logevent( $logstring );
               }
               $target_file_basename = '';
               $active_target_file = '';
               $target_file_contents = '';
               continue;
            }
            else {
               // try to open target file for processing
               $target_file_contents = $this->fix_eol( file_get_contents( $active_target_file ) );

               if( empty( $target_file_contents ) ) {
                  if( $fileoptional ) {
                     $logstring .= "{$admtext['optlocked']}&nbsp;<span class=\"msgapproved\">{$admtext['bypassed']}</span>";
                     $this->add_logevent( $logstring );
                  }
                  else {
                     $num_errors++;
                     $logstring .= "<span class=\"msgerror\">{$admtext['tgtlocked']}</span>";
                     $this->add_logevent( $logstring );
                  }
                  $target_file_basename = '';
                  $active_target_file = '';
                  $target_file_contents = '';
                  continue;
               }
               else {
                  $logstring .= "{$admtext['opened']}";
                  $this->add_logevent( $logstring );
                  continue;
               }
            }
         } // target
/*************************************************************************
         LOCATIONS
*************************************************************************/
         elseif( $tags[$j]['name'] == 'location' ) {
            $locations_installed++;
            $logstring = "$line {$tags[$j]['line']}: <span class=\"tag\">%location:%</span>&nbsp;";
            if( empty( $active_target_file) ) {
               if( $fileoptional ) {
                  $locations_installed--;
               }
               continue;
            }
            else {
            // target file is open and ready for modification
            $target_code = $tags[$j+1]['arg1'];
            $optag = $tags[$j+3]['raw']; // skip location %end:% tag
            $new_code = $tags[$j+4]['arg1'];
            $logstring .= "($optag)&nbsp;";
            }

            // count when replacement made in target file buffer
            $count = 0;

            // remove inserted locations
            if( $optag == "%insert:before%" || $optag == "%insert:after%" ) {
               // remove CRLF added when inserted
               if( $optag == "%insert:before%" )
                  $new_code .= "\r\n";
               else
                  $new_code = "\r\n".$new_code;

               // try to remove new code as given in cfg file with added CRLF
               $target_file_contents = str_replace(  $new_code, "", $target_file_contents, $count );
            } // inserted locations

				elseif( $optag == "%triminsert:before%" ) {
				   // try to remove code with all whitespace inserted per the cfg file
               $target_file_contents = str_replace( $new_code.$target_code, $target_code, $target_file_contents, $count );
				} // triminsert:before

				elseif( $optag == "%triminsert:after%" ) {
               // remove code with all whitespace inserted per the cfg file
               $target_file_contents = str_replace( $target_code.$new_code, $target_code, $target_file_contents, $count );
  				} // triminsert:after

				elseif( $optag == "%replace:%" || $optag == "%trimreplace:%" ) {
				   // try to replace inserted code including all whitespace per the cfg file
					$target_file_contents = str_replace( $new_code, $target_code, $target_file_contents, $count );
				} // replace & trimreplace

            // log results of implementing opcode
            if( $count ) {
               $locations_removed++;
               $logstring .= "<span class=\"msgapproved\">{$admtext['removed']}</span>";
               $this->add_logevent( $logstring );
               continue;
            }
            else {
               $logstring .= "<span class=\"msgapproved\">{$admtext['alreadyrem']}</span>";
               $this->add_logevent( $logstring );
               continue;
            }
         } // location
/*************************************************************************
         FILE OPERATIONS
*************************************************************************/
         elseif( $tags[$j]['name'] == 'copyfile' || $tags[$j]['name'] == 'copyfile2' || $tags[$j]['name'] == 'newfile' ) {
            $optional = '';
            if( false !== strpos($tags[$j]['arg1'], "@" ) ) {
               $optional = '@';
               $tags[$j]['arg1'] = str_replace( "@", "", $tags[$j]['arg1'] );
            }
            else {
               $files_installed++;
            }
            if( $tags[$j]['name'] == 'copyfile' ) {
               //$destination_path = $this->rootpath . pathinfo( $tags[$j]['arg1'], PATHINFO_BASENAME );
               $dest_path = pathinfo( $tags[$j]['arg1'], PATHINFO_BASENAME );
               $destination_path = $this->rootpath . $dest_path;
            }
            elseif( $tags[$j]['name'] == 'copyfile2' ) {
               $dest_path = $tags[$j]['arg2'];
               $destination_path = $this->rootpath.$tags[$j]['arg2'];
            }
            elseif( $tags[$j]['name'] == 'newfile') {
               $dest_path = $tags[$j]['arg1'];
               $destination_path = $this->rootpath.$tags[$j]['arg1'];
            }
            $logstring = "$line {$tags[$j]['line']}: %<span class=\"tag\">{$tags[$j]['name']}:</span><span class=\"tgtfile\">$optional.$dest_path</span>%&nbsp;";

            if( !file_exists( $destination_path ) ) {
               if( !$optional ) {
                  $files_installed--;
               }
               $logstring .= "{$admtext['notinst']}&nbsp;<span class=\"msgapproved\">{$admtext['bypassed']}</span>";
               $this->add_logevent( $logstring );
               continue;
            }
      		elseif (!@unlink( $destination_path ) ) {
      		   if( $optional ) {
      		      $logstring .= "<span class=\"msgapproved\">{$admtext['cantremok']}</span>";
                  $this->add_logevent( $logstring );
                  continue;
      		   }
               else {
               // failed to delete required file
                  $num_errors++;
                  $logstring .= "<span class=\"hilighterr msgbold\">{$admtext['cantdel']}</span>";
             		$this->add_logevent( $logstring );
                  continue;
               }
         	}
            else {
               $files_removed++;
               $logstring .= "<span class=\"msgapproved\">{$admtext['deleted']}</span>";
               $this->add_logevent( $logstring );
               continue;
         	}
         }
      } // tags processing loop

      // Everything processed, flush last buffer to file
      if( !empty( $active_target_file ) && !empty( $target_file_contents ) ) {
         file_put_contents( $active_target_file, $target_file_contents );
      }
      if( !$num_errors ) {
         $status = self::MODREM;
         $class = "class=\"msgapproved\"";
      }
      else {
         $status = self::ERRORS;
         $class = "class=\"msgerror\"";
      }

      $this->add_logevent("<span class=\"msgbold\">{$admtext['toterrors']}:</span> $num_errors");
      $this->add_logevent("$locations_installed {$admtext['modsinst']}&nbsp;&nbsp;&nbsp;&nbsp; $locations_removed {$admtext['modsrem']}" );
      $this->add_logevent("$files_installed {$admtext['filesinst']}&nbsp;&nbsp;&nbsp;&nbsp;$files_removed {$admtext['filesrem']}" );
      $this->add_logevent("{$admtext['status']}: <span $class>{$admtext[$status]}</span>");

      $this->mod_status = $status;
      $this->write_eventlog();
      return ( $status == self::NOTINST || $status == self::MODREM );
   } // install

   public function batch_remove( $cfgpathlist ) {
      foreach( $cfgpathlist as $cfgpath ) {
         if( !$this->remove( $cfgpath ) ) {
            $this->batch->error = true;
         };
      }
      return !$this->batch_error;
   }
} // class modparser

?>